/***************************************************************************
                          errordlg.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "errordlg.h"

#include <qstring.h>
#include <qlabel.h>

ErrorDlg::ErrorDlg(const QString& title, const QString& l1, const QString& l2, const QString& l3,
		   QWidget* parent, const char *name, WFlags fl)
  : ErrorBaseDlg(parent, name, true, fl)
{
  setCaption(title);
  line1->setText(l1);
  line2->setText(l2);
  line3->setText(l3);
}

ErrorInexistingRef::ErrorInexistingRef(QString ref, QWidget* parent, const char *name, WFlags fl)
  : ErrorDlg("Error when choosing object",
	     "The name \"" + ref + "\" doesn't exist",
	     "Please choose another name", QString::null,
	     parent, name, fl)
{ }

ErrorNameConflict::ErrorNameConflict(QString conflict, QWidget* parent, const char *name, WFlags fl)
  : ErrorDlg("Error when choosing name",
	     "Name \"" + conflict + "\" conflict with another existing object.",
	     "Please choose another name or resolve the conflict.", QString::null,
	     parent, name, fl)
{ }

ErrorStillRef::ErrorStillRef(QString ref, QWidget* parent, const char *name, WFlags fl)
  : ErrorDlg("Error when deleting object",
	     "The object named \"" + ref + "\" is stilled referenced.",
	     "Break all references before deleting object.", QString::null,
	     parent, name, fl)
{ }

ErrorSetZone::ErrorSetZone(QWidget* parent, const char *name, WFlags fl)
  : ErrorDlg("Error when setting up zone",
	     "Please, verify  there are some selected cases",
	     QString::null, QString::null,
	     parent, name, fl)
{ }

ErrorZoneSet::ErrorZoneSet(QWidget* parent, const char *name, WFlags fl)
  : ErrorDlg("Error when setting up zone set",
	     "Whether you choose an invalid name",
	     "or you didn't choose some zone",
	     "Please verify this things next time",
	     parent, name, fl)
{ }

ErrorXmlSyntax::ErrorXmlSyntax(QWidget* parent, const char *name, WFlags fl)
  : ErrorDlg("Error when loading file",
	     "There is a syntax error in your XML file",
	     "Check its syntax before relaoding", QString::null,
	     parent, name, fl)
{ }

ErrorBadData::ErrorBadData(QWidget* parent, const char *name, WFlags fl)
  : ErrorDlg("Error when setting up an object",
	     "The data were incorrect",
	     "Verify the integrity of your data", QString::null,
	     parent, name, fl)
{ }

