/***************************************************************************
                          myzonesetedit.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "config.h"
#include "rcobject.h"

#include "myzonesetedit.h"
#include "functions.h"

#include <qlistbox.h>
#include <qlineedit.h>

#include <set>
#include <algorithm>

MyZoneSetEdit::MyZoneSetEdit(const QString& n, const set<QString>& zones, const set<QString>& used,
			     QWidget* parent, const char* name, bool modal, WFlags fl)
  : ZoneSetEdit(parent, name, modal, fl)
{
  set_name->setText(n);
  set<QString> l_unused;
  unused->clear();
  selected->clear();
  insert_iterator<set<QString> > ii(l_unused, l_unused.begin());
  set_difference(zones.begin(), zones.end(), used.begin(), used.end(), ii);
  set<QString>::const_iterator it_qs;
  for( it_qs = l_unused.begin() ; it_qs != l_unused.end() ; ++it_qs )
    {
      unused->insertItem( *it_qs );
    }
  for( it_qs = used.begin() ; it_qs != used.end() ; ++it_qs )
    {
      selected->insertItem( *it_qs );
    }
  unused->sort();
  selected->sort();
}

void MyZoneSetEdit::remove_zone()
{
  int item = selected->currentItem();
  if(item != -1)
    {
      QString text = selected->currentText();
      unused->insertItem(text);
      selected->removeItem(item);
      unused->sort();
    }
}

void MyZoneSetEdit::put_zone()
{
  int item = unused->currentItem();
  if(item != -1)
    {
      QString text = unused->currentText();
      selected->insertItem(text);
      unused->removeItem(item);
      selected->sort();
    }
}

void MyZoneSetEdit::accept()
{
  QListBoxItem *it = selected->firstItem();
  while(it != NULL)
    {
      used.insert(it->text());
      it = it->next();
    }
  QDialog::accept();
}

