/***************************************************************************
                          objectref.cc  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "objectref.h"

#include <algorithm>

Referencer::reference_id ObjectRef::refAsObject( const QString& name, SpaceObject* obj )
{
  Referencer::reference_id ref = refCenter->createRef( name, this );
  Object o;
  o.type = MAP;
  o.map = obj;
  objects[ ref ] = o;
  return ref;
}

Referencer::reference_id ObjectRef::refAsObject( const QString& name, ProgElement* obj )
{
  Referencer::reference_id ref = refCenter->createRef( name, this );
  Object o;
  o.type = PROG;
  o.prog = obj;
  objects[ ref ] = o;
  return ref;
}

void ObjectRef::bindObject( Referencer::reference_id ref )
{
  if( objects.find( ref ) == objects.end() )
    {
      throw InexistingRefException( ref );
    }
  refCenter->bind( ref );
}

void ObjectRef::eraseReference( Referencer::reference_id ref )
{
  if( ref == Referencer::null_reference )
    return;
  ref_type::iterator found = objects.find( ref );
  if( found == objects.end() )
    {
      throw InexistingRefException( ref );
    }
  refCenter->eraseRef( ref );
  objects.erase( found );
}

vector<Referencer::reference_id> ObjectRef::objRefs() const
{
  vector<Referencer::reference_id> result;
  ref_type::const_iterator it;
  for( it = objects.begin() ; it != objects.end() ; ++it )
    {
      result.push_back( it->first );
    }
  std::sort( result.begin(), result.end(), CmpRefsByName( refCenter ) );
  return result;
}

void ObjectRef::clear()
{
  objects.clear();
}

