/***************************************************************************
                          progelement.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "progelement.h"

#include "global.h"
#include "test_dlg.h"
#include "progref.h"
#include "progitems.h"
#include "progitems2.h"
#include "progitems3.h"
#include "errordlg.h"

#include <qpainter.h>
#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qlineedit.h>

#include <iostream>
#include <map>

using namespace std;

ProgElement::ProgElement(QListView* p, ProgRef* pref, QString ind, bool opt, QString n)
  : QListViewItem(p)
  , Optional(opt)
  , valid(false)
  , isUniq( false )
  , needed(!opt)
  , index(ind)
  , name(n)
  , progRef( pref )
{
  setOpen( true );
  setText(0, name);
}

ProgElement::ProgElement(ProgElement* p, ProgRef* pref, QString ind, bool opt, QString n)
  : QListViewItem(p)
  , Optional(opt)
  , valid(false)
  , isUniq( false )
  , needed(!opt)
  , index(ind)
  , name( n )
  , progRef( pref )
{
  setOpen( true );
  setText(0, name);
}

ProgElement::ProgElement()
  : QListViewItem( ( QListView* )NULL )
{ }

ProgElement::~ProgElement()
{
  try
    {
      breakReferences();
    }
  catch( ... )
    {
      cerr << "Error when breaking reference in destructor of " << name << endl;
    }
}

ProgElement* ProgElement::replaceWith( ProgElement *elmt )
{
  ProgElement *copy = elmt->getExactCopy( dynamic_cast<ProgElement*>( parent() ), index );
  copy->needed = needed;
  copy->isUniq = isUniq;
  if( isUniq || !Optional )
    {
      breakReferences();
      delete this;
    }
  else
    {
      index.append( "1" );
    }
  return copy;
}

ProgElement* ProgElement::getExactCopy( ProgElement* parent, QString index) const
{
  ProgElement *res = getCopy(parent, index);
  res->needed = needed;
  res->isUniq = isUniq;
  res->clearChilds();
  res->setOptionsFrom( this );
  ProgElement *child = dynamic_cast<ProgElement*>( firstChild() );
  while( child != NULL )
    {
      child->getExactCopy( res, child->index );
      child = dynamic_cast<ProgElement*>( child->nextSibling() );
    }
  return res;
}

void ProgElement::clearChilds()
{
  if( childCount() == 0 )
    {
      return;
    }
  delete firstChild();
  clearChilds();
}

bool ProgElement::isValidSubTree()
{
  if( !validItem() )
    {
      return false;
    }
  ProgElement* child = dynamic_cast<ProgElement*>( firstChild() );
  while( child != NULL)
    {
      if( !child->isValidSubTree() )
        {
          return false;
        }
      child = dynamic_cast<ProgElement*>( child->nextSibling() );
    }
  return true;
}

void ProgElement::addChildToXml( QDomDocument& doc, QDomElement& xml )
{
  QDomElement xml_child;
  QListViewItem *child;
  ProgElement *real_child;
  for( child = firstChild() ; child != NULL ; child = child->nextSibling() )
    {
      real_child = dynamic_cast<ProgElement*>( child );
      xml_child = real_child->getXmlElement( doc );
      xml.appendChild( xml_child );
    }
}

void ProgElement::transferRefs( ProgElement* source )
{
  transferRefsToChilds( source );
}

void ProgElement::transferRefsToChilds( ProgElement* source )
{
  QListViewItem *child, *child_src;
  ProgElement *real_child, *real_child_src;
  sortChildItems( 1,true );
  source->sortChildItems( 1,true );
  for( child = firstChild(), child_src = source->firstChild() ;
       ( child != NULL ) && ( child_src != NULL ) ;
       child = child->nextSibling(), child_src = child_src->nextSibling() )
    {
      real_child = dynamic_cast<ProgElement*>( child );
      real_child_src = dynamic_cast<ProgElement*>( child_src );
      real_child->transferRefs( real_child_src );
    }
}

QPixmap *ProgTest::image = NULL;
QPixmap *ProgTest::opt_image = NULL;

ProgTest::ProgTest(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString n)
  : ProgElement(parent, pref, index, opt, n)
  , switch_yes( false )
  , switch_no( false )
  , test_id( Referencer::null_reference )
{
  if( image == NULL )
    {
      image = new QPixmap(getEditorDir() + "cond.png");
      opt_image = new QPixmap(getEditorDir() + "cond_op.png");
    }
  updatePixmap();
}

void ProgTest::updatePixmap()
{
  if( Optional )
    {
      setPixmap( 0, *opt_image );
    }
  else
    {
      setPixmap( 0, *image );
    }
}

bool ProgTest::isSameClass(ProgElement* element) const
{
  ProgTest* test = dynamic_cast<ProgTest*>(element);
  return test != NULL;
}

ProgElement* ProgTest::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTest( parent, progRef, index, Optional );
}

TestDlg* ProgTest::getDlg( QWidget* parent, const char* name)
{
  TestDlg *dlg = new TestDlg(parent, name, true);
  dlg->TestRefConfig->hide();
  dlg->TestOpConfig->hide();
  dlg->TestEvalConfig->hide();
  dlg->switch_yes->setChecked( switch_yes );
  dlg->switch_no->setChecked( switch_no );
  dlg->name_test->setText( progRef->getName( Referencer::null_reference ) );
  return dlg;
}

void ProgTest::setDlg( TestDlg* dlg )
{
  QString nt = dlg->name_test->text();
  if( nt != "" )
    {
      if( test_id != Referencer::null_reference )
        {
          progRef->renameRef( test_id, nt );
        }
      else
        {
          test_id = progRef->refAsCond( nt, this );
        }
    }
  else
    {
      if( test_id != Referencer::null_reference )
        {
          progRef->eraseReference( test_id );
          test_id = Referencer::null_reference;
        }
    }
  switch_yes = dlg->switch_yes->isChecked();
  switch_no = dlg->switch_no->isChecked();
}

bool ProgTest::isReferenced()
{
  if( test_id == Referencer::null_reference )
    return ProgElement::isReferenced();
  if( progRef->refCenter->referenced( test_id ) )
    return true;
  else
    return ProgElement::isReferenced();
}

void ProgTest::breakReferences()
{
  if( test_id != Referencer::null_reference )
    {
      progRef->eraseReference( test_id );
      test_id = Referencer::null_reference;
    }
}

void ProgTest::setOptionsFrom(const ProgElement* copy )
{
  const ProgTest* o = dynamic_cast<const ProgTest*>( copy );
  if( o != NULL )
    {
     // name_test = o->name_test;
      switch_yes = o->switch_yes;
      switch_no = o->switch_no;
    }
}

void ProgTest::transferRefs( ProgElement* source )
{
  ProgTest* src = dynamic_cast<ProgTest*>( source );
  if( src != NULL )
    {
      test_id = src->test_id;
      src->test_id = Referencer::null_reference;
      ProgElement::transferRefs( source );
    }
}

void ProgTest::setFromXml( QDomElement& xml )
{
  QString yes, no;
  yes = xml.attribute( "switch_yes", "no" );
  no = xml.attribute( "switch_no", "no" );
  QString name_test = xml.attribute( "name", "" );
  if( name_test != "" )
    {
      try
        {
          test_id = progRef->refAsCond( name_test, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
        }
    }
  else
    {
      test_id = Referencer::null_reference;
    }
  switch_yes = ( yes.lower() == "yes" );
  switch_yes = ( no.lower() == "yes" );
}

QDomElement ProgTest::getXmlElement( QDomDocument& doc, QDomElement subCond )
{
  QDomElement xml = doc.createElement( "cond" );
  xml.setAttribute( "switch_yes",  switch_yes ? "yes" : "no"  );
  xml.setAttribute( "switch_no",  switch_no ? "yes" : "no" );
  if( test_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( test_id ) );
    }
  xml.appendChild( subCond );
  return xml;
}

ProgTest* ProgTest::getElement( QDomElement& xml, ProgElement* dummy_parent, ProgRef* pref )
{
  QDomElement xml_child = xml.firstChild().toElement();
  if( xml_child.isNull() )
    {
      cerr << "cond element has non-element child or no child" << endl;
      throw InvalidXmlException();
    }
  ProgTest *child = NULL;
  QString tag = xml_child.tagName();
  if( tag == "condref" )
    {
      child = new ProgTestCondRef( dummy_parent, pref, "" );
    }
  else if( tag == "and" )
    {
      child = new ProgTestAnd( dummy_parent, pref, "" );
    }
  else if( tag == "or" )
    {
      child = new ProgTestOr( dummy_parent, pref, "" );
    }
  else if( tag == "not" )
    {
      child = new ProgTestNot( dummy_parent, pref, "" );
    }
  else if( tag == "eval" )
    {
      child = new ProgTestEval( dummy_parent, pref, "" );
    }
  else if( tag == "onboard" )
    {
      child = new ProgTestOnboard( dummy_parent, pref, "" );
    }
  else if( tag == "state" )
    {
      child = new ProgTestState( dummy_parent, pref, "" );
    }
  else if( tag == "position" )
    {
      child = new ProgTestPosition( dummy_parent, pref, "" );
    }
  else if( tag == "turntype" )
    {
      child = new ProgTestTurntype( dummy_parent, pref, "" );
    }
  else if( tag == "inf" )
    {
      child = new ProgTestInf( dummy_parent, pref, "" );
    }
  else if( tag == "sup" )
    {
      child = new ProgTestSup( dummy_parent, pref, "" );
    }
  else if( tag == "equal" )
    {
      child = new ProgTestEqual( dummy_parent, pref, "" );
    }
  else if( tag == "true" )
    {
      child = new ProgTestTrue( dummy_parent, pref, "" );
    }
  else if( tag == "false" )
    {
      child = new ProgTestFalse( dummy_parent, pref, "" );
    }
  if( child != NULL )
    {
      child->setFromXml( xml );
    }
  return child;
}


QPixmap *ProgValue::image = NULL;
QPixmap *ProgValue::opt_image = NULL;

ProgValue::ProgValue(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString n)
  : ProgElement(parent, pref, index, opt, n)
{
  if( image == NULL )
    {
      image = new QPixmap(getEditorDir() + "value.png");
      opt_image = new QPixmap(getEditorDir() + "value_op.png");
    }
  updatePixmap();
}

void ProgValue::updatePixmap()
{
  if( Optional )
    {
      setPixmap( 0, *opt_image );
    }
  else
    {
      setPixmap( 0, *image );
    }
}

bool ProgValue::isSameClass(ProgElement* element) const
{
  ProgValue* test = dynamic_cast<ProgValue*>(element);
  return test != NULL;
}

ProgElement* ProgValue::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValue( parent, progRef, index, Optional );
}

ProgValue* ProgValue::getElement( QDomElement& xml, ProgElement* dummy_parent, ProgRef* pref )
{
  ProgValue* child = NULL;
  QString tag = xml.tagName();
  if( tag == "random" )
    {
      child = new ProgValueRandom( dummy_parent, pref, "" );
    }
  else if( tag == "turn" )
    {
      child = new ProgValueTurn( dummy_parent, pref, "" );
    }
  else if( tag == "leaved" )
    {
      child = new ProgValueLeaved( dummy_parent, pref, "" );
    }
  else if( tag == "AP" )
    {
      child = new ProgValueAP( dummy_parent, pref, "" );
    }
  else if( tag == "taken" )
    {
      child = new ProgValueTaken( dummy_parent, pref, "" );
    }
  else if( tag == "size" )
    {
      child = new ProgValueSize( dummy_parent, pref, "" );
    }
  else if( tag == "dead" )
    {
      child = new ProgValueDead( dummy_parent, pref, "" );
    }
  else if( tag == "blockedentry" )
    {
      child = new ProgValueBlockedentry( dummy_parent, pref, "" );
    }
  else if( tag == "value" )
    {
      child = new ProgValueValue( dummy_parent, pref, "" );
    }
  else if( tag == "product" )
    {
      child = new ProgValueProduct( dummy_parent, pref, "" );
    }
  else if( tag == "sum" )
    {
      child = new ProgValueSum( dummy_parent, pref, "" );
    }
  if( child != NULL )
    {
      child->setFromXml( xml );
    }
  return child;
}

static const char* list_of_values[] = 
{
  "random",
  "turn",
  "leaved",
  "AP",
  "taken",
  "size",
  "dead",
  "blockedentry",
  "value",
  "product",
  "sum"
};

//static const char** end_list_of_values = list_of_values + sizeof( list_of_values );
static size_t size_list_of_values = sizeof( list_of_values ) / sizeof( const char* );

bool ProgValue::isValueTag( const QString& tag )
{
  size_t pos;
  for( pos = 0 ; pos < size_list_of_values ; ++pos )
    {
      if( tag == list_of_values[ pos ] )
        return true;
    }
  return false;
}

QPixmap *ProgAction::image = NULL;
QPixmap *ProgAction::opt_image = NULL;

ProgAction::ProgAction(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString n)
  : ProgElement(parent, pref, index, opt, n)
{
  if( image == NULL )
    {
      image = new QPixmap(getEditorDir() + "action.png");
      opt_image = new QPixmap(getEditorDir() + "action_op.png");
    }
  updatePixmap();
}

void ProgAction::transferRefs( ProgElement* source )
{
  ProgAction* src = dynamic_cast<ProgAction*>( source );
  if( src != NULL )
    {
      ProgElement::transferRefs( source );
    }
}

void ProgAction::updatePixmap()
{
  if( Optional )
    {
      setPixmap( 0, *opt_image );
    }
  else
    {
      setPixmap( 0, *image );
    }
}

bool ProgAction::isSameClass(ProgElement* element) const
{
  ProgAction* test = dynamic_cast<ProgAction*>(element);
  return test != NULL;
}

ProgElement* ProgAction::getCopy(ProgElement* parent, QString index) const
{
  return new ProgAction( parent, progRef, index, Optional );
}

bool ProgAction::isActionTag( const QString& tag )
{
  if( ( tag == "place" ) || ( tag == "destroy" ) )
    {
      return true;
    }
  return false;
}

ProgAction* ProgAction::getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref )
{
  QDomElement xml_child = xml.firstChild().toElement();
  if( xml_child.isNull() )
    {
      cerr << "Action element must have a child element" << endl;
      throw InvalidXmlException();
    }
  ProgAction *child = NULL;
  QString tag = xml_child.tagName();
  if( tag == "cond" )
    {
      xml_child = xml_child.nextSibling().toElement();
      if( xml_child.isNull() )
        {
          cerr << "Action element must have a child element after the cond one" << endl;
          throw InvalidXmlException();
        }
      tag = xml_child.tagName();
    }
  if( tag == "place" )
    {
      child = new ProgActionPlace( parent, pref, "" );
    }
  else if( tag == "destroy" )
    {
      child = new ProgActionDestroy( parent, pref, "" );
    }
  if( child != NULL )
    {
      child->setFromXml( xml );
    }
  return child;
}

void ProgAction::setFromXml( QDomElement& xml )
{
  QDomElement child = xml.firstChild().toElement();
  if( child.tagName() == "cond" )
    {
      ProgElement* par = dynamic_cast<ProgElement*>( parent() );
      sortChildItems( 1, true );
      enforceSortOrder ();
      ProgTest* test = ProgTest::getElement( child, par, progRef );
      ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
      while( dynamic_cast<ProgTest*>( real_child ) == NULL )
        {
          if( real_child == NULL )
            {
              cerr << "Error, no appropriate child to action place object" << endl;
              throw InvalidDataException();
            }
          real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
        }
      real_child = real_child->replaceWith( test );
      real_child->transferRefs( test );
      delete test;
    }
}

QDomElement ProgAction::getXmlElement( QDomDocument& doc, QDomElement subAction )
{
  QDomElement xml = doc.createElement( "action" );
  QDomElement cond = dynamic_cast<ProgElement*>( firstChild() )->getXmlElement( doc );
  xml.appendChild( xml );
  xml.appendChild( subAction );
  return xml;
}

QPixmap *ProgObject::image = NULL;
QPixmap *ProgObject::opt_image = NULL;

ProgObject::ProgObject(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString n)
  : ProgElement(parent,pref, index, opt, n)
{
  if( image == NULL )
    {
      image = new QPixmap(getEditorDir() + "object.png");
      opt_image = new QPixmap(getEditorDir() + "object_op.png");
    }
  updatePixmap();
}

void ProgObject::updatePixmap()
{
  if( Optional )
    {
      setPixmap( 0, *opt_image );
    }
  else
    {
      setPixmap( 0, *image );
    }
}

bool ProgObject::isSameClass(ProgElement* element) const
{
  ProgObject* test = dynamic_cast<ProgObject*>(element);
  return test != NULL;
}

ProgElement* ProgObject::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObject( parent, progRef, index, Optional );
}

ProgObjectPlace::ProgObjectPlace(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString n)
  : ProgObjectFind(parent,pref, index, opt, n)
{ }

bool ProgObjectPlace::isSameClass(ProgElement* element) const
{
  ProgObjectPlace* test = dynamic_cast<ProgObjectPlace*>(element);
  return test != NULL;
}

ProgElement* ProgObjectPlace::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectPlace( parent, progRef, index, Optional );
}

static const char* list_of_place_objects[] = 
{
  "object",
  "cat",
  "extensible",
  "squad"
};

//static const char** end_list_of_place_objects = list_of_place_objects + sizeof( list_of_place_objects );
static size_t size_list_of_place_objects = sizeof( list_of_place_objects ) / sizeof( const char* );

bool ProgObjectPlace::isObjectPlaceTag( const QString& tag )
{
  if( ProgObjectInSquad::isObjectSquadTag( tag ) )
    {
      return true;
    }
  size_t pos;
  for( pos = 0 ; pos < size_list_of_place_objects; ++pos )
    {
      if( tag == list_of_place_objects[ pos ] )
        return true;
    }
  return false;
}

ProgObjectPlace* ProgObjectPlace::getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref )
{
  ProgObjectPlace* child = ProgObjectInSquad::getElement( xml, parent, pref );
  if( child == NULL )
    {
      QString tag = xml.tagName();
      if( tag == "object" )
        {
          child = new ProgObjectObject( parent, pref, "" );
        }
      else if( tag == "cat" )
        {
          child = new ProgObjectCat( parent, pref, "" );
        }
      else if( tag == "extensible" )
        {
          child = new ProgObjectExtensible( parent, pref, "" );
        }
      else if( tag == "squad" )
        {
          child = new ProgObjectSquad( parent, pref, "" );
        }
      if( child != NULL )
        {
          child->setFromXml( xml );
        }
    }
  return child;
}

ProgObjectFind::ProgObjectFind(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString n)
  : ProgObject(parent,pref, index, opt, n)
{ }

bool ProgObjectFind::isSameClass(ProgElement* element) const
{
  ProgObjectFind* test = dynamic_cast<ProgObjectFind*>(element);
  return test != NULL;
}

ProgElement* ProgObjectFind::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectFind( parent, progRef, index, Optional );
}

static const char* list_of_find_objects[] = 
{
  "door",
  "bulkhead",
  "flame",
  "entry"
};

//static const char** end_list_of_find_objects = list_of_find_objects + sizeof( list_of_find_objects );
static size_t size_list_of_find_objects = sizeof( list_of_find_objects ) / sizeof( const char* );

bool ProgObjectFind::isObjectFindTag( const QString& tag )
{
  if( ProgObjectPlace::isObjectPlaceTag( tag ) )
    {
      return true;
    }
  size_t pos;
  for( pos = 0 ; pos < size_list_of_find_objects ; ++pos )
    {
      if( tag == list_of_find_objects[ pos ] )
        return true;
    }
  return false;
}

ProgObjectFind* ProgObjectFind::getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref )
{
  ProgObjectFind* child = ProgObjectPlace::getElement( xml, parent, pref );
  if( child == NULL )
    {
      QString tag = xml.tagName();
      if( tag == "door" )
        {
        }
      else if( tag == "bulkhead" )
        {
        }
      else if( tag == "flame" )
        {
        }
      else if( tag == "entry" )
        {
        }
      if( child != NULL )
        {
          child->setFromXml( xml );
        }
    }
  return child;
}

ProgObjectInSquad::ProgObjectInSquad(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString n)
  : ProgObjectPlace(parent,pref, index, opt, n)
{ }

bool ProgObjectInSquad::isSameClass(ProgElement* element) const
{
  ProgObjectInSquad* test = dynamic_cast<ProgObjectInSquad*>(element);
  return test != NULL;
}

ProgElement* ProgObjectInSquad::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectInSquad( parent, progRef, index, Optional );
}

static const char* list_of_squad_objects[] = 
{
  "sergeant",
  "flamer",
  "terminator",
  "genestealer",
  "blip"
};

//static const char** end_list_of_squad_objects = list_of_squad_objects + sizeof( list_of_squad_objects );
static size_t size_list_of_squad_objects = sizeof( list_of_squad_objects ) / sizeof( const char* );

bool ProgObjectInSquad::isObjectSquadTag( const QString& tag )
{
  size_t pos;
  for( pos = 0 ; pos < size_list_of_squad_objects ; ++pos )
    {
      if( tag == list_of_squad_objects[ pos ] )
        return true;
    }
  return false;
}

ProgObjectInSquad* ProgObjectInSquad::getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref )
{
  ProgObjectInSquad *child = NULL;
  QString tag = xml.tagName();
  if( tag == "sergeant" )
    {
      child = new ProgObjectSergeant( parent, pref, "" );
    }
  else if( tag == "flamer" )
    {
      child = new ProgObjectFlamer( parent, pref, "" );
    }
  else if( tag == "terminator" )
    {
      child = new ProgObjectTerminator( parent, pref, "" );
    }
  else if( tag == "genestealer" )
    {
      child = new ProgObjectGenestealer( parent, pref, "" );
    }
  else if( tag == "blip" )
    {
      child = new ProgObjectBlip( parent, pref, "" );
    }
  if( child != NULL )
    {
      child->setFromXml( xml );
    }
  return child;
}

