/***************************************************************************
                          progelement.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _PROGELEMENT_HH_
#define _PROGELEMENT_HH_

#include "config.h"

#include "spaceref.h"

#include <qpixmap.h>
#include <qstring.h>
#include <qlistview.h>
#include <qdom.h>

using namespace std;

#define SIZE_CASE 104

class ProgRef;

class ProgElement : public QListViewItem
{
public:
  ProgElement(QListView* parent, ProgRef* pref, QString index, bool opt, QString name);
  ProgElement(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name);
  ProgElement();
  virtual ~ProgElement();
  bool optional() const {return Optional;}
  bool isNeeded() const { return needed; }
  bool uniq() const { return isUniq; }

  void setUniq( bool uniq = true ) { isUniq = uniq; }
  void setOptional( bool opt = true ) { Optional = opt; updatePixmap(); }

  virtual QString key (int, bool) const { return index; }
  //virtual QString text ( int ) const { return name; }
  virtual bool isSameClass(ProgElement* ) const { return false; }
  virtual ProgElement* getCopy(ProgElement* , QString ) const { return NULL; }
  const QString& getText() { return name; }
  ProgElement* replaceWith( ProgElement* elmt );
  ProgElement* getExactCopy( ProgElement* parent, QString index) const;
  void clearChilds();
  virtual void updatePixmap() {}
  bool isValidSubTree();
  virtual void config( QWidget* = NULL, const char* = "" ) {}
  bool isReferenced() { return false; }
  virtual void breakReferences() {}
  virtual void setOptionsFrom(const ProgElement*) {};
  void setIndex( QString new_index ) { index = new_index; }

  virtual QDomElement getXmlElement( QDomDocument& /*doc*/ ) { return QDomElement(); }
  virtual void setFromXml( QDomElement& /*xml*/ ) {}
  void addChildToXml( QDomDocument& doc, QDomElement& xml );
  virtual void transferRefs( ProgElement* source );

protected:
  void transferRefsToChilds( ProgElement* source );
  virtual bool validItem() { return false; }
  bool Optional;
  bool valid;
  bool isUniq;
  bool needed;
  QString index;
  QString name;
  ProgRef *progRef;
};

class TestDlg;

class ProgTest : public ProgElement
{
public:
  ProgTest(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Test" );

  virtual bool isSameClass(ProgElement* element) const;
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void updatePixmap();

  bool isReferenced();
  void breakReferences();
  virtual void setOptionsFrom(const ProgElement* copy );
  virtual void setFromXml( QDomElement& xml );
  virtual QDomElement getXmlElement( QDomDocument& doc, QDomElement subCond );
  static ProgTest* getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref );
  virtual void transferRefs( ProgElement* source );

protected:
  bool switch_yes, switch_no;
  Referencer::reference_id test_id;

  TestDlg* getDlg( QWidget* parent = NULL, const char* name = "");
  void setDlg( TestDlg* dlg );
  static QPixmap *image, *opt_image;
};

class ProgValue : public ProgElement
{
public:
  ProgValue(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Value" );

  virtual bool isSameClass(ProgElement* element) const;
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void updatePixmap();

  static ProgValue* getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref );
  static bool isValueTag( const QString& tag );

protected:
  static QPixmap *image, *opt_image;
};

class ProgAction : public ProgElement
{
public:
  ProgAction(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Action" );

  virtual bool isSameClass(ProgElement* element) const;
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void updatePixmap();
  static bool isActionTag( const QString& tag );
  static ProgAction* getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref );
  virtual void transferRefs( ProgElement* source );
  virtual void setFromXml( QDomElement& xml );
  virtual QDomElement getXmlElement( QDomDocument& doc, QDomElement subCond );

protected:
  static QPixmap *image, *opt_image;
};

class ProgObject : public ProgElement
{
public:
  ProgObject(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Object" );

  virtual bool isSameClass(ProgElement* element) const;
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void updatePixmap();

protected:
  enum
  {
    OBJECT_FIND = 1 << 0,
    OBJECT_PLACE = 1 << 1,
    OBJECT_SQUAD = 1 << 2
  };
  static QPixmap *image, *opt_image;
};

class ProgObjectFind : public ProgObject
{
public:
  ProgObjectFind(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Object to find" );

  virtual bool isSameClass(ProgElement* element) const;
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  static bool isObjectFindTag( const QString& tag );
  static ProgObjectFind* getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref );
};

class ProgObjectPlace : public ProgObjectFind
{
public:
  ProgObjectPlace(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Object to place" );

  virtual bool isSameClass(ProgElement* element) const;
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  static bool isObjectPlaceTag( const QString& tag );
  static ProgObjectPlace* getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref );
};

class ProgObjectInSquad : public ProgObjectPlace
{
public:
  ProgObjectInSquad(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Object for squad" );

  virtual bool isSameClass(ProgElement* element) const;
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  static bool isObjectSquadTag( const QString& tag );
  static ProgObjectInSquad* getElement( QDomElement& xml, ProgElement* parent, ProgRef* pref );
};

#endif
