/***************************************************************************
                          progitems.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "progitems.h"
#include "global.h"
#include "test_dlg.h"
#include "progref.h"
#include "positionconfig.h"
#include "stateconfig.h"
#include "onboardconfig.h"
#include "turntypeconfig.h"
#include "functions.h"
#include "errordlg.h"

#include <qpixmap.h>
#include <qgroupbox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>

#include <iostream>
#include <functional>
#include <algorithm>

using namespace std;

QPixmap* ProgTestCondRef::icon = NULL;

ProgTestCondRef::ProgTestCondRef(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
  , test_ref( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_ref.png" );
    }
  setPixmap( 0, *icon );
}

ProgTestCondRef::~ProgTestCondRef()
{
  try
    {
      if( test_ref != Referencer::null_reference )
        {
          progRef->unbind( test_ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgTestCondRef" << endl;
    }
}

void ProgTestCondRef::config( QWidget* parent, const char* name )
{
  TestDlg* dlg = getDlg(parent, name);
  vector<Referencer::reference_id> refs = progRef->condRefs();
  size_t ind = 0;
  for( size_t i = 0 ; i < refs.size() ; i++ )
    {
      dlg->name_ref->insertItem( progRef->getName( refs[ i ] ) );
      if( refs[ i ] == test_ref )
        ind = i;
    }
  dlg->name_ref->setCurrentItem( ind );
  //dlg->name_ref->setEditText( name_ref );
  dlg->TestRefConfig->show();
  if( dlg->exec() == QDialog::Accepted )
    {
      setDlg(dlg);
      QString nr = dlg->name_ref->currentText();
      if( test_ref != Referencer::null_reference )
        {
          progRef->unbind( test_ref );
        }
      if( nr != "" )
        {
          test_ref = progRef->getId( nr );
          progRef->bindCond( test_ref );
        }
    }
}

ProgElement* ProgTestCondRef::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestCondRef( parent, progRef, index );
}

void ProgTestCondRef::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestCondRef *o = dynamic_cast<const ProgTestCondRef*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
      test_ref = o->test_ref;
      if( test_ref != Referencer::null_reference )
        {
          progRef->bind_nofail( test_ref );
        }
    }
}

QDomElement ProgTestCondRef::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "condref" );
  xml.setAttribute( "name", progRef->getName( test_ref ) );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestCondRef::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  QDomElement child = xml.firstChild().toElement();
  QString name_ref = child.attribute( "name", "" );
  if( name_ref == "" )
    {
      cerr << "Error, condref must have attribute 'name'" << endl;
      ErrorXmlSyntax dlg;
      dlg.exec();
      return;
    }
  test_ref = progRef->getId_nofail( name_ref );
  progRef->bind_nofail( test_ref );
}

QPixmap* ProgTestAnd::icon = NULL;

ProgTestAnd::ProgTestAnd(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
  , test_all( false )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_and.png" );
    }
  setPixmap( 0, *icon );
  new ProgTest( this, pref, "1" );
  new ProgTest( this, pref, "2" );
  new ProgTest( this, pref, "3", true );
  sort();
}

ProgElement* ProgTestAnd::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestAnd( parent, progRef, index );
}

void ProgTestAnd::config( QWidget* parent, const char* name )
{
  TestDlg* dlg = getDlg(parent, name);
  dlg->TestOpConfig->show();
  dlg->test_all->setChecked( test_all );
  if( dlg->exec() == QDialog::Accepted )
    {
      setDlg(dlg);
      test_all = dlg->test_all->isChecked();
    }
}

void ProgTestAnd::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestAnd *o = dynamic_cast<const ProgTestAnd*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
      test_all = o->test_all;
    }
}

QDomElement ProgTestAnd::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "and" );
  xml.setAttribute( "test_all", test_all ? "yes" : "no"  );
  addChildToXml( doc, xml );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestAnd::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  while( firstChild() != NULL )
    {
      delete firstChild();
    }
  int num_child = 0;
  QDomElement child = xml.firstChild().toElement();
  test_all = ( child.attribute( "test_all", "no" ) == "yes" );
  child = child.firstChild().toElement();
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( child.tagName() != "cond" )
        {
          cerr << "'and' element child must be 'cond'" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgTest::getElement( child, this, progRef );
      if( new_child != NULL )
        {
          num_child++;
          new_child->setIndex( QString::number( num_child ) );
        }
      child = child.nextSibling().toElement();
    }
  while( num_child < 2 )
    {
      num_child++;
      new ProgTest( this, progRef, QString::number( num_child ) );
    }
  num_child++;
  new ProgTest( this, progRef, QString::number( num_child ), true );
}

QPixmap* ProgTestOr::icon = NULL;

ProgTestOr::ProgTestOr(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
  , test_all( false )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_or.png" );
    }
  setPixmap( 0, *icon );
  new ProgTest( this, pref, "1" );
  new ProgTest( this, pref, "2" );
  new ProgTest( this, pref, "3", true );
  sort();
}

ProgElement* ProgTestOr::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestOr( parent, progRef, index );
}

void ProgTestOr::config( QWidget* parent, const char* name )
{
  TestDlg* dlg = getDlg(parent, name);
  dlg->TestOpConfig->show();
  dlg->test_all->setChecked( test_all );
  if( dlg->exec() == QDialog::Accepted )
    {
      setDlg(dlg);
      test_all = dlg->test_all->isChecked();
    }
}

void ProgTestOr::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestOr *o = dynamic_cast<const ProgTestOr*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
      test_all = o->test_all;
    }
}

QDomElement ProgTestOr::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "or" );
  xml.setAttribute( "test_all", test_all ? "yes" : "no" );
  addChildToXml( doc, xml );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestOr::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  while( firstChild() != NULL )
    {
      delete firstChild();
    }
  int num_child = 0;
  QDomElement child = xml.firstChild().toElement();
  test_all = ( child.attribute( "test_all", "no" ) == "yes" );
  child = child.firstChild().toElement();
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( child.tagName() != "cond" )
        {
          cerr << "'or' element child must be 'cond'" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgTest::getElement( child, this, progRef );
      if( new_child != NULL )
        {
          num_child++;
          new_child->setIndex( QString::number( num_child ) );
        }
      child = child.nextSibling().toElement();
    }
  while( num_child < 2 )
    {
      num_child++;
      new ProgTest( this, progRef, QString::number( num_child ) );
    }
  num_child++;
  new ProgTest( this, progRef, QString::number( num_child ), true );
}

QPixmap* ProgTestNot::icon = NULL;

ProgTestNot::ProgTestNot(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_not.png" );
    }
  setPixmap( 0, *icon );
  new ProgTest( this, pref, "1" );
}

ProgElement* ProgTestNot::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestNot( parent, progRef, index );
}

void ProgTestNot::config( QWidget* parent, const char* name )
{
  TestDlg* dlg = getDlg(parent, name);
  if( dlg->exec() == QDialog::Accepted )
    {
      setDlg(dlg);
    }
}

void ProgTestNot::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestNot* o = dynamic_cast<const ProgTestNot*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
    }
}

QDomElement ProgTestNot::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "not" );
  addChildToXml( doc, xml );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestNot::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  QDomElement child = xml.firstChild().toElement();
  child = child.firstChild().toElement();
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  ProgElement* new_child;
  ProgElement* par = dynamic_cast<ProgElement*>( parent() );
  if( ! child.isNull() )
    {
      if( child.tagName() != "cond" )
        {
          cerr << "and element child must be 'cond'" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgTest::getElement( child, par, progRef );
      if( new_child != NULL )
        {
          real_child = real_child->replaceWith( new_child );
          real_child->transferRefs( new_child );
          delete new_child;
        }
    }
}

QPixmap* ProgTestEval::icon = NULL;

ProgTestEval::ProgTestEval(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
  , eval_id( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_eval.png" );
    }
  setPixmap( 0, *icon );
  new ProgValue( this, pref, "1" );
}

ProgElement* ProgTestEval::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestEval( parent, progRef, index );
}

void ProgTestEval::config( QWidget* parent, const char* name )
{
  TestDlg* dlg = getDlg(parent, name);
  dlg->TestEvalConfig->show();
  dlg->switch_no->setDisabled( true );
  dlg->name_test->setDisabled( true );
  dlg->name_eval->setText( progRef->getName( eval_id ) );
  if( dlg->exec() == QDialog::Accepted )
    {
      QString name_eval = dlg->name_eval->text();
      setDlg(dlg);
      if( name_eval != "" )
        {
          if( eval_id != Referencer::null_reference )
            {
              progRef->renameRef( eval_id, name_eval );
            }
          else
            {
              eval_id = progRef->refAsEval( name_eval, this );
            }
        }
      else
        {
          if( eval_id != Referencer::null_reference )
            {
              progRef->eraseReference( eval_id );
              eval_id = Referencer::null_reference;
            }
        }
    }
}

bool ProgTestEval::isReferenced()
{
  if( eval_id == Referencer::null_reference )
    {
      return ProgTest::isReferenced();
    }
  if( progRef->refCenter->referenced( eval_id ) )
    return true;
  else
    return ProgTest::isReferenced();
}

void ProgTestEval::breakReferences()
{
  progRef->eraseReference( eval_id );
  eval_id = Referencer::null_reference;
}

QDomElement ProgTestEval::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "eval" );
  if( eval_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( eval_id ) );
    }
  addChildToXml( doc, xml );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestEval::transferRefs( ProgElement* source )
{
  ProgTestEval* src = dynamic_cast<ProgTestEval*>( source );
  if( src != NULL )
    {
      eval_id = src->eval_id;
      src->eval_id = Referencer::null_reference;
      ProgTest::transferRefs( source );
    }
}

void ProgTestEval::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  QDomElement child = xml.firstChild().toElement();
  QString name_eval = child.attribute( "name", "" );
  if( name_eval == "" )
    {
      cerr << "eval element must have attribute 'name'" << endl;
      ErrorXmlSyntax dlg;
      dlg.exec();
      eval_id = Referencer::null_reference;
    }
  else
    {
      try
        {
          eval_id = progRef->refAsEval( name_eval, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          eval_id = Referencer::null_reference;
        }
    }
  child = child.firstChild().toElement();
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  ProgElement* par = dynamic_cast<ProgElement*>( parent() );
  ProgElement *new_child;
  if( !child.isNull() )
    {
      if( !ProgValue::isValueTag( child.tagName() ) )
        {
          cerr << "Child of 'eval' must be a value" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgValue::getElement( child, par, progRef );
      if( new_child != NULL )
        {
          real_child = real_child->replaceWith( new_child );
          delete new_child;
        }
    }
}

QPixmap* ProgTestTrue::icon = NULL;

ProgTestTrue::ProgTestTrue(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_true.png" );
    }
  setPixmap( 0, *icon );
}

ProgElement* ProgTestTrue::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestTrue( parent, progRef, index );
}

QDomElement ProgTestTrue::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "true" );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestTrue::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
}

QPixmap* ProgTestFalse::icon = NULL;

ProgTestFalse::ProgTestFalse(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_false.png" );
    }
  setPixmap( 0, *icon );
}

ProgElement* ProgTestFalse::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestFalse( parent, progRef, index );
}

QDomElement ProgTestFalse::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "false" );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestFalse::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
}

QPixmap* ProgTestOnboard::icon = NULL;

ProgTestOnboard::ProgTestOnboard(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
  , type( "marine" )
  , unit_id( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_onboard.png" );
    }
  setPixmap( 0, *icon );
}

void ProgTestOnboard::changeName()
{
  name = "Any " + type + " onboard ?";
  setText( 0, name );
}

void ProgTestOnboard::breakReferences()
{
  progRef->eraseReference( unit_id );
  unit_id = Referencer::null_reference;
}

ProgElement* ProgTestOnboard::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestOnboard( parent, progRef, index );
}

void ProgTestOnboard::config( QWidget* parent, const char* name )
{
  OnboardConfig *dlg = new OnboardConfig(parent, name, true);
  dlg->switch_yes->setChecked( switch_yes );
  dlg->switch_no->setChecked( switch_no );
  dlg->name_test->setText( progRef->getName( test_id ) );
  size_t i;
  for( i = 0 ; i < dlg->type->count() ; ++i )
    {
      if( dlg->type->text( i ) == type )
        {
          dlg->type->setCurrentItem( i );
          break;
        }
    }
  dlg->unit_name->setText( progRef->getName( unit_id ) );
  if( dlg->exec() == QDialog::Accepted )
    {
      QString bn = dlg->unit_name->text();
      if( bn != "" )
        {
          if( unit_id != Referencer::null_reference )
            {
              progRef->renameRef( unit_id, bn );
            }
          else
            {
              unit_id = progRef->refAsObject( bn, this );
            }
        }
      else
        {
          if( unit_id != Referencer::null_reference )
            {
              progRef->eraseReference( unit_id );
              unit_id = Referencer::null_reference;
            }
        }

      type = dlg->type->currentText();
      switch_yes = dlg->switch_yes->isChecked();
      switch_no = dlg->switch_no->isChecked();
      QString nt = dlg->name_test->text();
      if( nt != "" )
        {
          if( test_id != Referencer::null_reference )
            {
              progRef->renameRef( test_id, nt );
            }
          else
            {
              test_id = progRef->refAsCond( nt, this );
            }
        }
      else
        {
          if( test_id != Referencer::null_reference )
            {
              progRef->eraseReference( test_id );
              test_id = Referencer::null_reference;
            }
        }
      changeName();
    }
}

void ProgTestOnboard::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestOnboard *o = dynamic_cast<const ProgTestOnboard*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
      type = o->type;
      changeName();
    }
}

QDomElement ProgTestOnboard::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "onboard" );
  xml.setAttribute( "type", type );
  if( unit_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( unit_id ) );
    }
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestOnboard::transferRefs( ProgElement* source )
{
  ProgTestOnboard* src = dynamic_cast<ProgTestOnboard*>( source );
  if( src != NULL )
    {
      unit_id = src->unit_id;
      src->unit_id = Referencer::null_reference;
      ProgTest::transferRefs( source );
      changeName();
    }
}

void ProgTestOnboard::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  type = xml.attribute( "type", "marine" );
  QString unit_name = xml.attribute( "name", "" );
  if( unit_name != "" )
    {
      try
        {
          unit_id = progRef->refAsObject( unit_name, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          unit_id = Referencer::null_reference;
        }
    }
  changeName();
}

QPixmap* ProgTestState::icon = NULL;

ProgTestState::ProgTestState(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
  , entire( false )
  , state( "onfire" )
  , zone( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_state.png" );
    }
  setPixmap( 0, *icon );
}

void ProgTestState::changeName()
{
  if( zone != Referencer::null_reference )
    {
      name = ( entire ? "Entire z" : "Z" ) + ( "one " + progRef->getName_nofail( zone ) ) + " is " + state;
    }
  else
    {
      name = "State";
    }
  setText( 0, name );
}

ProgTestState::~ProgTestState()
{
  try
    {
      if( zone != Referencer::null_reference )
        {
          progRef->unbind( zone );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgTestState" << endl;
    }
}

ProgElement* ProgTestState::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestState( parent, progRef, index );
}

void ProgTestState::config( QWidget* parent, const char* name )
{
  StateConfig *dlg = new StateConfig(parent, name, true);
  dlg->switch_yes->setChecked( switch_yes );
  dlg->switch_no->setChecked( switch_no );
  dlg->entire->setCurrentItem( entire ? 0 : 1 );
  size_t i;
  for( i = 0 ; i < dlg->state->count() ; ++i )
    {
      if( dlg->state->text( i ) == state )
        {
          dlg->state->setCurrentItem( i );
          break;
        }
    }
  vector<Referencer::reference_id> zones = progRef->zoneRefs();
  size_t pos = 0;
  for( i = 0 ; i < zones.size() ; ++i )
    {
      dlg->zone->insertItem( progRef->getName( zones[ i ] ) );
      if( zones[ i ] == zone )
        {
          pos = i;
        }
    }
  dlg->zone->setCurrentItem( pos );
  if( dlg->exec() == QDialog::Accepted )
    {
      entire = ( dlg->entire->currentText() == "Yes" );
      state = dlg->state->currentText();
      switch_yes = dlg->switch_yes->isChecked();
      switch_no = dlg->switch_no->isChecked();
      QString nz = dlg->zone->currentText();
      if( nz.isNull() )
        {
          nz = "";
        }
      if( nz != progRef->getName( zone ) )
        {
          if( zone != Referencer::null_reference )
            {
              progRef->unbind( zone );
            }
          if( nz != "" )
            {
              try
                {
                  zone = progRef->getId( nz );
                }
              catch( ... )
                {
                  progRef->bindZone( zone );
                  throw;
                }
              progRef->bindZone( zone );
            }
        }
      QString nt = dlg->name_test->text();
      if( nt != "" )
        {
          if( test_id != Referencer::null_reference )
            {
              progRef->renameRef( test_id, nt );
            }
          else
            {
              test_id = progRef->refAsCond( nt, this );
            }
        }
      else
        {
          if( test_id != Referencer::null_reference )
            {
              progRef->eraseReference( test_id );
              test_id = Referencer::null_reference;
            }
        }
      changeName();
    }
}

void ProgTestState::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestState *o = dynamic_cast<const ProgTestState*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
      entire = o->entire;
      state = o->state;
      zone = o->zone;
      if( zone != Referencer::null_reference )
        {
          progRef->bind_nofail( zone );
        }
      changeName();
    }
}

QDomElement ProgTestState::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "state" );
  xml.setAttribute( "entire", entire ? "yes" : "no" );
  xml.setAttribute( "state", state );
  progRef->saveZoneRef( progRef->getName( zone ), xml, doc );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestState::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  QDomElement child = xml.firstChild().toElement();
  entire = ( child.attribute( "entire", "yes" ) == "yes" );
  state = child.attribute( "state", "onfire" );
  QDomElement elmt = child.firstChild().toElement();
  if( !elmt.isNull() )
    {
      zone = progRef->xmlZone( elmt );
      progRef->bindZone( zone );
    }
  changeName();
}


QPixmap* ProgTestPosition::icon = NULL;

ProgTestPosition::ProgTestPosition(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
  , type( "marine" )
  , obj_id( Referencer::null_reference )
  , zone( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_position.png" );
    }
  setPixmap( 0, *icon );
}

void ProgTestPosition::changeName()
{
  if( zone != Referencer::null_reference )
    {
      name = type + " is in zone " + progRef->getName_nofail( zone );
    }
  else
    {
      name = "Position";
    }
  setText( 0, name );
}

void ProgTestPosition::breakReferences()
{
  progRef->eraseReference( obj_id );
  obj_id = Referencer::null_reference;
}

ProgTestPosition::~ProgTestPosition()
{
  try
    {
      if( zone != Referencer::null_reference )
        {
          progRef->unbind( zone );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgTestPosition" << endl;
    }
}

ProgElement* ProgTestPosition::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestPosition( parent, progRef, index );
}

void ProgTestPosition::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestPosition *o = dynamic_cast<const ProgTestPosition*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
      type = o->type;
      zone = o->zone;
      if( zone != Referencer::null_reference )
        {
          progRef->bind_nofail( zone );
        }
      changeName();
    }
}

void ProgTestPosition::transferRefs( ProgElement* source )
{
  ProgTestPosition* src = dynamic_cast<ProgTestPosition*>( source );
  if( src != NULL )
    {
      obj_id = src->obj_id;
      src->obj_id = Referencer::null_reference;
      ProgTest::transferRefs( source );
      changeName();
    }
}


void ProgTestPosition::config( QWidget* parent, const char* name )
{
  PositionConfig *dlg = new PositionConfig(parent, name, true);
  dlg->switch_yes->setChecked( switch_yes );
  dlg->switch_no->setChecked( switch_no );
  dlg->name_test->setText( progRef->getName( test_id ) );
  dlg->name_obj->setText( progRef->getName( obj_id ) );
  vector<Referencer::reference_id> zones = progRef->zoneRefs();
  size_t pos = 0;
  for( size_t i = 0 ; i < zones.size() ; ++i )
    {
      dlg->zone->insertItem( progRef->getName( zones[ i ] ) );
      if( zones[ i ] == zone )
        {
          pos = i;
        }
    }
  dlg->zone->setCurrentItem( pos );
  if( dlg->exec() == QDialog::Accepted )
    {
      switch_yes = dlg->switch_yes->isChecked();
      switch_no = dlg->switch_no->isChecked();
      QString nz = dlg->zone->currentText();
      if( nz != progRef->getName( zone ) )
        {
          if( zone != Referencer::null_reference )
            {
              progRef->unbind( zone );
            }
          if( nz != "" )
            {
              try
                {
                  zone = progRef->getId( nz );
                }
              catch( ... )
                {
                  progRef->bindZone( zone );
                  throw;
                }
              progRef->bindZone( zone );
            }
        }

      QString no = dlg->name_obj->text();
      if( no != "" )
        {
          if( obj_id != Referencer::null_reference )
            {
              progRef->renameRef( obj_id, no );
            }
          else
            {
              obj_id = progRef->refAsObject( no, this );
            }
        }
      else
        {
          if( obj_id != Referencer::null_reference )
            {
              progRef->eraseReference( obj_id );
              obj_id = Referencer::null_reference;
            }
        }

      QString nt = dlg->name_test->text();
      if( nt != "" )
        {
          if( test_id != Referencer::null_reference )
            {
              progRef->renameRef( test_id, nt );
            }
          else
            {
              test_id = progRef->refAsCond( nt, this );
            }
        }
      else
        {
          if( test_id != Referencer::null_reference )
            {
              progRef->eraseReference( test_id );
              test_id = Referencer::null_reference;
            }
        }
      changeName();
    }
}

QDomElement ProgTestPosition::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "position" );
  xml.setAttribute( "type", type );
  progRef->saveZoneRef( progRef->getName( zone ), xml, doc );
  if( obj_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( obj_id ) );
    }
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestPosition::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  QDomElement child = xml.firstChild().toElement();
  type = child.attribute( "type", "marine" );
  QString name = child.attribute( "name", "" );
  if( name != "" )
    {
      obj_id = progRef->refAsObject( name, this );
    }
  QDomElement elmt = child.firstChild().toElement();
  if( !elmt.isNull() )
    {
      zone = progRef->xmlZone( elmt );
      progRef->bindZone( zone );
    }
  changeName();
}


QPixmap* ProgTestInf::icon = NULL;

ProgTestInf::ProgTestInf(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_inf.png" );
    }
  setPixmap( 0, *icon );
  new ProgValue( this, pref, "1" );
  new ProgValue( this, pref, "2" );
  sort();
}

ProgElement* ProgTestInf::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestInf( parent, progRef, index );
}

void ProgTestInf::config( QWidget* parent, const char* name )
{
  TestDlg* dlg = getDlg(parent, name);
  if( dlg->exec() == QDialog::Accepted )
    {
      setDlg(dlg);
    }
}

void ProgTestInf::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestInf* o = dynamic_cast<const ProgTestInf*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
    }
}

QDomElement ProgTestInf::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "inf" );
  addChildToXml( doc, xml );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestInf::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  while( firstChild() != NULL )
    {
      delete firstChild();
    }
  int num_child = 0;
  QDomElement child = xml.firstChild().toElement();
  child = child.firstChild().toElement();
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( ! ProgValue::isValueTag( child.tagName() ) )
        {
          cerr << "'inf' element childs must be values" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgValue::getElement( child, this, progRef );
      if( new_child != NULL )
        {
          num_child++;
          new_child->setIndex( QString::number( num_child ) );
        }
      child = child.nextSibling().toElement();
    }
  while( num_child < 2 )
    {
      num_child++;
      new ProgValue( this, progRef, QString::number( num_child ) );
    }
}


QPixmap* ProgTestSup::icon = NULL;

ProgTestSup::ProgTestSup(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_sup.png" );
    }
  setPixmap( 0, *icon );
  new ProgValue( this, pref, "1" );
  new ProgValue( this, pref, "2" );
  sort();
}

ProgElement* ProgTestSup::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestSup( parent, progRef, index );
}

void ProgTestSup::config( QWidget* parent, const char* name )
{
  TestDlg* dlg = getDlg(parent, name);
  if( dlg->exec() == QDialog::Accepted )
    {
      setDlg(dlg);
    }
}

void ProgTestSup::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestSup* o = dynamic_cast<const ProgTestSup*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
    }
}

QDomElement ProgTestSup::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "sup" );
  addChildToXml( doc, xml );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestSup::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  while( firstChild() != NULL )
    {
      delete firstChild();
    }
  int num_child = 0;
  QDomElement child = xml.firstChild().toElement();
  child = child.firstChild().toElement();
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( ! ProgValue::isValueTag( child.tagName() ) )
        {
          cerr << "'sup' element childs must be values" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgValue::getElement( child, this, progRef );
      if( new_child != NULL )
        {
          num_child++;
          new_child->setIndex( QString::number( num_child ) );
        }
      child = child.nextSibling().toElement();
    }
  while( num_child < 2 )
    {
      num_child++;
      new ProgValue( this, progRef, QString::number( num_child ) );
    }
}


QPixmap* ProgTestEqual::icon = NULL;

ProgTestEqual::ProgTestEqual(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_equal.png" );
    }
  setPixmap( 0, *icon );
  new ProgValue( this, pref, "1" );
  new ProgValue( this, pref, "2" );
  sort();
}

ProgElement* ProgTestEqual::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestEqual( parent, progRef, index );
}

void ProgTestEqual::config( QWidget* parent, const char* name )
{
  TestDlg* dlg = getDlg(parent, name);
  if( dlg->exec() == QDialog::Accepted )
    {
      setDlg(dlg);
    }
}

void ProgTestEqual::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestEqual* o = dynamic_cast<const ProgTestEqual*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
    }
}

QDomElement ProgTestEqual::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "equal" );
  addChildToXml( doc, xml );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestEqual::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  while( firstChild() != NULL )
    {
      delete firstChild();
    }
  int num_child = 0;
  QDomElement child = xml.firstChild().toElement();
  child = child.firstChild().toElement();
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( ! ProgValue::isValueTag( child.tagName() ) )
        {
          cerr << "'equal' element childs must be values" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgValue::getElement( child, this, progRef );
      if( new_child != NULL )
        {
          num_child++;
          new_child->setIndex( QString::number( num_child ) );
        }
      child = child.nextSibling().toElement();
    }
  while( num_child < 2 )
    {
      num_child++;
      new ProgValue( this, progRef, QString::number( num_child ) );
    }
}

QPixmap* ProgTestTurntype::icon = NULL;

ProgTestTurntype::ProgTestTurntype(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgTest( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "cond_turntype.png" );
    }
  setPixmap( 0, *icon );
}

ProgElement* ProgTestTurntype::getCopy(ProgElement* parent, QString index) const
{
  return new ProgTestTurntype( parent, progRef, index );
}

void ProgTestTurntype::config( QWidget* parent, const char* name )
{
  TurntypeConfig *dlg = new TurntypeConfig(parent, name, true);
  dlg->switch_yes->setChecked( switch_yes );
  dlg->switch_no->setChecked( switch_no );
  dlg->name_test->setText( progRef->getName( test_id ) );
  for( size_t i = 0 ; i < dlg->type->count() ; ++i )
    {
      if( dlg->type->text( i ) == type )
        {
          dlg->type->setCurrentItem( i );
          break;
        }
    }
  if( dlg->exec() == QDialog::Accepted )
    {
      type = dlg->type->currentText();
      switch_yes = dlg->switch_yes->isChecked();
      switch_no = dlg->switch_no->isChecked();
      QString nt = dlg->name_test->text();
      if( nt != "" )
        {
          if( test_id != Referencer::null_reference )
            {
              progRef->renameRef( test_id, nt );
            }
          else
            {
              test_id = progRef->refAsCond( nt, this );
            }
        }
      else
        {
          if( test_id != Referencer::null_reference )
            {
              progRef->eraseReference( test_id );
              test_id = Referencer::null_reference;
            }
        }

    }
}

void ProgTestTurntype::setOptionsFrom(const ProgElement* copy)
{
  const ProgTestTurntype* o = dynamic_cast<const ProgTestTurntype*>( copy );
  if( o != NULL )
    {
      ProgTest::setOptionsFrom( copy );
      type = o->type;
    }
}

QDomElement ProgTestTurntype::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "turntype" );
  xml.setAttribute( "type", type );
  addChildToXml( doc, xml );
  return ProgTest::getXmlElement( doc, xml );
}

void ProgTestTurntype::setFromXml( QDomElement& xml )
{
  ProgTest::setFromXml( xml );
  QDomElement child = xml.firstChild().toElement();
  type = child.attribute( "type", "marine" );
}

