/***************************************************************************
                          progitems.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROGITEMS_HH
#define PROGITEMS_HH

#include "progelement.h"

#include <qstring.h>

class ProgTestCondRef : public ProgTest
{
public:
  ProgTestCondRef(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Reference on condition");
  ~ProgTestCondRef();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id test_ref;
};

class ProgTestAnd : public ProgTest
{
public:
  ProgTestAnd(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "And");
  ~ProgTestAnd() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  bool test_all;
};

class ProgTestOr : public ProgTest
{
public:
  ProgTestOr(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Or" );
  ~ProgTestOr() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  bool test_all;
};

class ProgTestNot : public ProgTest
{
public:
  ProgTestNot(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Not" );
  ~ProgTestNot() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgTestEval : public ProgTest
{
public:
  ProgTestEval(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Eval" );
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  bool isReferenced();
  void breakReferences();
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id eval_id;
};

class ProgTestTrue : public ProgTest
{
public:
  ProgTestTrue(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "True" );
  ~ProgTestTrue() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgTestFalse : public ProgTest
{
public:
  ProgTestFalse(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "False" );
  ~ProgTestFalse() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgTestOnboard : public ProgTest
{
public:
  ProgTestOnboard(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Onboard" );
  ~ProgTestOnboard() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual void transferRefs( ProgElement* source );
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  void breakReferences();
  void changeName();
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  QString type;
  Referencer::reference_id unit_id;
};

class ProgTestState : public ProgTest
{
public:
  ProgTestState(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "State" );
  ~ProgTestState();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  void changeName();
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  bool entire;
  QString state;
  Referencer::reference_id zone;
};

class ProgTestPosition : public ProgTest
{
public:
  ProgTestPosition(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Position" );
  ~ProgTestPosition();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
  void breakReferences();
  void changeName();
protected:
  virtual bool validItem() { return true; }
  QString type;
  Referencer::reference_id obj_id;
  Referencer::reference_id zone;
  static QPixmap* icon;
};

class ProgTestInf : public ProgTest
{
public:
  ProgTestInf(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Inf" );
  ~ProgTestInf() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgTestSup : public ProgTest
{
public:
  ProgTestSup(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Sup" );
  ~ProgTestSup() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgTestEqual : public ProgTest
{
public:
  ProgTestEqual(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Equal" );
  ~ProgTestEqual() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgTestTurntype : public ProgTest
{
public:
  ProgTestTurntype(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Turn type" );
  ~ProgTestTurntype() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  QString type;
};

#endif // PROGITEMS_HH
