/***************************************************************************
                          progitems2.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "progitems2.h"
#include "global.h"
#include "progref.h"
#include "errordlg.h"

#include <qspinbox.h>
#include <qcombobox.h>

#include "randomconfig.h"
#include "leavedconfig.h"
#include "apconfig.h"
#include "takenconfig.h"
#include "sizeconfig.h"
#include "deadconfig.h"
#include "valueconfig.h"

#include <vector>
#include <iostream>

using namespace std;

QPixmap* ProgValueRandom::icon = NULL;

ProgValueRandom::ProgValueRandom(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
  , max( 6 )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_random.png" );
    }
  setPixmap( 0, *icon );
}

void ProgValueRandom::changeName()
{
  name = "Random(" + QString::number(max) + ")";
  setText(0, name);
}

ProgElement* ProgValueRandom::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueRandom( parent, progRef, index );
}

void ProgValueRandom::config( QWidget* parent, const char* name )
{
  RandomConfig dlg( parent, name, true );
  dlg.max->setValue( max );
  if( dlg.exec() == QDialog::Accepted )
    {
      max = dlg.max->value();
      changeName();
    }
}

void ProgValueRandom::setOptionsFrom(const ProgElement* copy)
{
  const ProgValueRandom* o = dynamic_cast<const ProgValueRandom*>( copy );
  if( o != NULL )
    {
      max = o->max;
      changeName();
    }
}

QDomElement ProgValueRandom::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "random" );
  xml.setAttribute( "max", QString::number( max ) );
  return xml;
}

void ProgValueRandom::setFromXml( QDomElement& xml )
{
  max = xml.attribute( "max", "6" ).toInt();
  changeName();
}

QPixmap* ProgValueTurn::icon = NULL;

ProgValueTurn::ProgValueTurn(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_turn.png" );
    }
  setPixmap( 0, *icon );
}

ProgElement* ProgValueTurn::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueTurn( parent, progRef, index );
}

void ProgValueTurn::config( QWidget* parent, const char* name )
{
}

void ProgValueTurn::setOptionsFrom(const ProgElement* copy)
{
}

QDomElement ProgValueTurn::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "turn" );
  return xml;
}

void ProgValueTurn::setFromXml( QDomElement& xml )
{
}

QPixmap* ProgValueLeaved::icon = NULL;

ProgValueLeaved::ProgValueLeaved(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
  , type( "marine" )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_leaved.png" );
    }
  setPixmap( 0, *icon );
}

void ProgValueLeaved::changeName()
{
  name = "Leaved " + type;
  setText(0, name);
}

ProgElement* ProgValueLeaved::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueLeaved( parent, progRef, index );
}

void ProgValueLeaved::config( QWidget* parent, const char* name )
{
  LeavedConfig dlg( parent, name, true );
  for( size_t i = 0 ; i < dlg.type->count() ; ++i )
    {
      if( dlg.type->text( i ) == type )
        {
          dlg.type->setCurrentItem( i );
          break;
        }
    }
  if( dlg.exec() == QDialog::Accepted )
    {
      type = dlg.type->currentText();
      changeName();
    }
}

void ProgValueLeaved::setOptionsFrom(const ProgElement* copy)
{
  const ProgValueLeaved* o = dynamic_cast<const ProgValueLeaved*>( copy );
  if( o != NULL )
    {
      type = o->type;
      changeName();
    }
}

QDomElement ProgValueLeaved::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "leaved" );
  xml.setAttribute( "type", type );
  return xml;
}

void ProgValueLeaved::setFromXml( QDomElement& xml )
{
  type = xml.attribute( "type", "marine" );
  changeName();
}


QPixmap* ProgValueAP::icon = NULL;

ProgValueAP::ProgValueAP(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
  , op( "max" )
  , ref( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_ap.png" );
    }
  setPixmap( 0, *icon );
}

void ProgValueAP::changeName()
{
  if( ref != Referencer::null_reference)
    {
      name = "AP of " + progRef->getName_nofail(ref);
    }
  else
    {
      name = "AP";
    }
  setText(0, name);
}

ProgValueAP::~ProgValueAP()
{
  if( ref != Referencer::null_reference )
    {
      try
        {
          progRef->unbind( ref );
        }
      catch( ... )
        {
          cerr << "Unexpected exception in ~ProgValueAP" << endl;
        }
    }
}

ProgElement* ProgValueAP::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueAP( parent, progRef, index );
}

void ProgValueAP::config( QWidget* parent, const char* name )
{
  APConfig dlg( parent, name, true );
  size_t i;
  for( i = 0 ; i < dlg.op->count() ; ++i )
    {
      if( dlg.op->text( i ) == op )
        {
          dlg.op->setCurrentItem( i );
          break;
        }
    }
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos = 0;
  for( i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos = i;
        }
    }
  dlg.ref->setCurrentItem( pos );
  if( dlg.exec() == QDialog::Accepted )
    {
      op = dlg.op->currentText();
      QString nr = dlg.ref->currentText();
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      changeName();
    }
}

void ProgValueAP::setOptionsFrom(const ProgElement* copy)
{
  const ProgValueAP* o = dynamic_cast<const ProgValueAP*>( copy );
  if( o != NULL )
    {
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
      op = o->op;
      changeName();
    }
}

QDomElement ProgValueAP::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "AP" );
  xml.setAttribute( "op", op );
  xml.setAttribute( "ref", progRef->getName( ref ) );
  return xml;
}

void ProgValueAP::setFromXml( QDomElement& xml )
{
  op = xml.attribute( "op", "max" );
  ref = progRef->getId_nofail( xml.attribute( "ref", "" ) );
  if( ref != Referencer::null_reference )
    {
      progRef->bind_nofail( ref );
    }
  changeName();
}


QPixmap* ProgValueTaken::icon = NULL;

ProgValueTaken::ProgValueTaken(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
  , ref( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_taken.png" );
    }
  setPixmap( 0, *icon );
}

void ProgValueTaken::changeName()
{
  /*
  if( ref != Referencer::null_reference)
    {
      name = progRef->getName_nofail(ref) + " is taken";
    }
  else
    {
      name = "Taken object";
    }
    */
      name = "Taken object";
  setText(0, name);
}

ProgValueTaken::~ProgValueTaken()
{
  if( ref != Referencer::null_reference )
    {
      try
        {
          progRef->unbind( ref );
        }
      catch( ... )
        {
          cerr << "Unexpected exception in ~ProgValueTaken" << endl;
        }
    }
}

ProgElement* ProgValueTaken::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueTaken( parent, progRef, index );
}

void ProgValueTaken::config( QWidget* parent, const char* name )
{
  TakenConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos = i;
        }
    }
  dlg.ref->setCurrentItem( pos );
  if( dlg.exec() == QDialog::Accepted )
    {
      QString nr = dlg.ref->currentText();
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      changeName();
    }
}

void ProgValueTaken::setOptionsFrom(const ProgElement* copy)
{
  const ProgValueTaken* o = dynamic_cast<const ProgValueTaken*>( copy );
  if( o != NULL )
    {
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
      changeName();
    }
}

QDomElement ProgValueTaken::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "taken" );
  xml.setAttribute( "ref", progRef->getName( ref ) );
  return xml;
}

void ProgValueTaken::setFromXml( QDomElement& xml )
{
  ref = progRef->getId_nofail( xml.attribute( "ref", "" ) );
  if( ref != Referencer::null_reference )
    {
      progRef->bind_nofail( ref );
    }
  changeName();
}

QPixmap* ProgValueSize::icon = NULL;

ProgValueSize::ProgValueSize(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
  , ref( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_size.png" );
    }
  setPixmap( 0, *icon );
}

void ProgValueSize::changeName()
{
  if( ref != Referencer::null_reference)
    {
      name = "Size of set " + progRef->getName_nofail(ref);
    }
  else
    {
      name = "Size of set";
    }
  setText(0, name);
}

ProgValueSize::~ProgValueSize()
{
  if( ref != Referencer::null_reference )
    {
      try
        {
          progRef->unbind( ref );
        }
      catch( ... )
        {
          cerr << "Unexpected exception in ~ProgValueSize" << endl;
        }
    }
}

ProgElement* ProgValueSize::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueSize( parent, progRef, index );
}

void ProgValueSize::config( QWidget* parent, const char* name )
{
  SizeConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos = i;
        }
    }
  dlg.ref->setCurrentItem( pos );
  if( dlg.exec() == QDialog::Accepted )
    {
      QString nr = dlg.ref->currentText();
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      changeName();
    }
}

void ProgValueSize::setOptionsFrom(const ProgElement* copy)
{
  const ProgValueSize* o = dynamic_cast<const ProgValueSize*>( copy );
  if( o != NULL )
    {
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
      changeName();
    }
}

QDomElement ProgValueSize::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "size" );
  xml.setAttribute( "ref", progRef->getName( ref ) );
  return xml;
}

void ProgValueSize::setFromXml( QDomElement& xml )
{
  QString ref_name = xml.attribute( "ref", "" );
  ref = progRef->getId_nofail( ref_name );
  if( ref != Referencer::null_reference )
    {
      progRef->bind_nofail( ref );
    }
  changeName();
}

QPixmap* ProgValueDead::icon = NULL;

ProgValueDead::ProgValueDead(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
  , type( "marine" )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_dead.png" );
    }
  setPixmap( 0, *icon );
}

void ProgValueDead::changeName()
{
  name = "Dead " + type;
  setText(0, name);
}

ProgElement* ProgValueDead::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueDead( parent, progRef, index );
}

void ProgValueDead::config( QWidget* parent, const char* name )
{
  DeadConfig dlg( parent, name, true );
  for( size_t i = 0 ; i < dlg.type->count() ; ++i )
    {
      if( dlg.type->text( i ) == type )
        {
          dlg.type->setCurrentItem( i );
          break;
        }
    }
  if( dlg.exec() == QDialog::Accepted )
    {
      type = dlg.type->currentText();
      changeName();
    }
}

void ProgValueDead::setOptionsFrom(const ProgElement* copy)
{
  const ProgValueDead* o = dynamic_cast<const ProgValueDead*>( copy );
  if( o != NULL )
    {
      type = o->type;
      changeName();
    }
}

QDomElement ProgValueDead::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "dead" );
  xml.setAttribute( "type", type );
  return xml;
}

void ProgValueDead::setFromXml( QDomElement& xml )
{
  type = xml.attribute( "type", "marine" );
  changeName();
}

QPixmap* ProgValueBlockedentry::icon = NULL;

ProgValueBlockedentry::ProgValueBlockedentry(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_blockedentry.png" );
    }
  setPixmap( 0, *icon );
}

ProgElement* ProgValueBlockedentry::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueBlockedentry( parent, progRef, index );
}

void ProgValueBlockedentry::config( QWidget* parent, const char* name )
{
}

void ProgValueBlockedentry::setOptionsFrom(const ProgElement* copy)
{
}

QDomElement ProgValueBlockedentry::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "turn" );
  return xml;
}

void ProgValueBlockedentry::setFromXml( QDomElement& xml )
{
}

QPixmap* ProgValueValue::icon = NULL;

ProgValueValue::ProgValueValue(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
  , number( 0 )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_random.png" );
    }
  setPixmap( 0, *icon );
}

void ProgValueValue::changeName()
{
  name = QString::number(number);
  setText(0, name);
}

ProgElement* ProgValueValue::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueValue( parent, progRef, index );
}

void ProgValueValue::config( QWidget* parent, const char* name )
{
  ValueConfig dlg( parent, name, true );
  dlg.number->setValue( number );
  if( dlg.exec() == QDialog::Accepted )
    {
      number = dlg.number->value();
      changeName();
    }
}

void ProgValueValue::setOptionsFrom(const ProgElement* copy)
{
  const ProgValueValue* o = dynamic_cast<const ProgValueValue*>( copy );
  if( o != NULL )
    {
      number = o->number;
      changeName();
    }
}

QDomElement ProgValueValue::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "value" );
  xml.setAttribute( "number", QString::number( number ) );
  return xml;
}

void ProgValueValue::setFromXml( QDomElement& xml )
{
  number = xml.attribute( "number", "0" ).toInt();
  changeName();
}

QPixmap* ProgValueProduct::icon = NULL;

ProgValueProduct::ProgValueProduct(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_product.png" );
    }
  setPixmap( 0, *icon );
  new ProgValue( this, pref, "1" );
  new ProgValue( this, pref, "2" );
  new ProgValue( this, pref, "3", true );
}

ProgElement* ProgValueProduct::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueProduct( parent, progRef, index );
}

void ProgValueProduct::config( QWidget* parent, const char* name )
{
}

void ProgValueProduct::setOptionsFrom(const ProgElement* copy)
{
}

QDomElement ProgValueProduct::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "product" );
  addChildToXml( doc, xml );
  return xml;
}

void ProgValueProduct::setFromXml( QDomElement& xml )
{
  while( firstChild() != NULL )
    {
      delete firstChild();
    }
  int num_child = 0;
  QDomElement child = xml.firstChild().toElement();
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( ! ProgValue::isValueTag( child.tagName() ) )
        {
          cerr << "'product' element child must be a value" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgValue::getElement( child, this, progRef );
      if( new_child != NULL )
        {
          num_child++;
          new_child->setIndex( QString::number( num_child ) );
        }
      child = child.nextSibling().toElement();
    }
  while( num_child < 2 )
    {
      num_child++;
      new ProgValue( this, progRef, QString::number( num_child ) );
    }
  num_child++;
  new ProgValue( this, progRef, QString::number( num_child ), true );
}

QPixmap* ProgValueSum::icon = NULL;

ProgValueSum::ProgValueSum(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgValue( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "value_sum.png" );
    }
  setPixmap( 0, *icon );
  new ProgValue( this, pref, "1" );
  new ProgValue( this, pref, "2" );
  new ProgValue( this, pref, "3", true );
}

ProgElement* ProgValueSum::getCopy(ProgElement* parent, QString index) const
{
  return new ProgValueSum( parent, progRef, index );
}

void ProgValueSum::config( QWidget* parent, const char* name )
{
}

void ProgValueSum::setOptionsFrom(const ProgElement* copy)
{
}

QDomElement ProgValueSum::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "sum" );
  addChildToXml( doc, xml );
  return xml;
}

void ProgValueSum::setFromXml( QDomElement& xml )
{
  while( firstChild() != NULL )
    {
      delete firstChild();
    }
  int num_child = 0;
  QDomElement child = xml.firstChild().toElement();
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( ! ProgValue::isValueTag( child.tagName() ) )
        {
          cerr << "'sum' element child must be a value" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgValue::getElement( child, this, progRef );
      if( new_child != NULL )
        {
          num_child++;
          new_child->setIndex( QString::number( num_child ) );
        }
      child = child.nextSibling().toElement();
    }
  while( num_child < 2 )
    {
      num_child++;
      new ProgValue( this, progRef, QString::number( num_child ) );
    }
  num_child++;
  new ProgValue( this, progRef, QString::number( num_child ), true );
}


