/***************************************************************************
                          progitems2.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROGITEMS2_HH
#define PROGITEMS2_HH

#include "spaceref.h"
#include "progelement.h"

class ProgValueRandom : public ProgValue
{
public:
  ProgValueRandom(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Random");
  ~ProgValueRandom() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  void changeName();
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  int max;
};

class ProgValueTurn : public ProgValue
{
public:
  ProgValueTurn(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Turns");
  ~ProgValueTurn() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgValueLeaved : public ProgValue
{
public:
  ProgValueLeaved(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Leaved units");
  ~ProgValueLeaved() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  void changeName();
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  QString type;
};

class ProgValueAP : public ProgValue
{
public:
  ProgValueAP(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "AP");
  ~ProgValueAP();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  void changeName();
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  QString op;
  Referencer::reference_id ref;
};

class ProgValueTaken : public ProgValue
{
public:
  ProgValueTaken(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Taken object");
  ~ProgValueTaken();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  void changeName();
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id ref;
};

class ProgValueSize : public ProgValue
{
public:
  ProgValueSize(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Size of set");
  ~ProgValueSize();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  void changeName();
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id ref;
};

class ProgValueDead : public ProgValue
{
public:
  ProgValueDead(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Dead");
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  void changeName();
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  QString type;
};

class ProgValueBlockedentry : public ProgValue
{
public:
  ProgValueBlockedentry(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Blocked entries");
  ~ProgValueBlockedentry() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgValueValue : public ProgValue
{
public:
  ProgValueValue(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Value");
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  void changeName();
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  int number;
};

class ProgValueProduct : public ProgValue
{
public:
  ProgValueProduct(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Product");
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgValueSum : public ProgValue
{
public:
  ProgValueSum(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Sum");
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

#endif // PROGITEMS2_HH

