/***************************************************************************
                          progitems3.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "progitems3.h"
#include "progref.h"
#include "errordlg.h"

#include <qlineedit.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qradiobutton.h>

#include "destroyconfig.h"
#include "placeconfig.h"
#include "terminatorconfig.h"
#include "flamerconfig.h"
#include "sergeantconfig.h"
#include "genestealerconfig.h"
#include "blipconfig.h"
#include "squadconfig.h"
#include "objectconfig.h"
#include "catconfig.h"
#include "extensibleconfig.h"

#include <iostream>

using namespace std;

static const char* dir_name[] = 
  { "west", "east", "south", "north", "none" }
;

QPixmap* ProgActionPlace::icon = NULL;

ProgActionPlace::ProgActionPlace(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgAction( parent, pref, index, opt, name )
  , number( 1 )
  , alone( false )
  , zoneset( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "action_place.png" );
    }
  setPixmap( 0, *icon );
  ProgTest* test = new ProgTest( this, pref, "1", true );
  test->setUniq( true );
  new ProgObjectPlace( this, pref, "2" );
}

ProgActionPlace::~ProgActionPlace()
{
  try
    {
      if( zoneset != Referencer::null_reference )
        {
          progRef->unbind( zoneset );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgActionPlace" << endl;
    }
}

ProgElement* ProgActionPlace::getCopy(ProgElement* parent, QString index) const
{
  return new ProgActionPlace( parent, progRef, index );
}

void ProgActionPlace::transferRefs( ProgElement* source )
{
  ProgActionPlace* src = dynamic_cast<ProgActionPlace*>( source );
  if( src != NULL )
    {
      ProgAction::transferRefs( source );
    }
}

void ProgActionPlace::config( QWidget* parent, const char* name )
{
  PlaceConfig dlg( parent, name, true );
  dlg.number->setValue( number );
  dlg.alone->setCurrentItem( alone ? 1 : 0 );
  vector<Referencer::reference_id> zone_sets = progRef->zonesetRefs();
  size_t pos = 0;
  for( size_t i = 0 ; i < zone_sets.size() ; ++i )
    {
      dlg.zoneset->insertItem( progRef->getName( zone_sets[ i ] ) );
      if( zone_sets[ i ] == zoneset )
        {
          pos = i;
        }
    }
  dlg.zoneset->setCurrentItem( pos );

  if( dlg.exec() == QDialog::Accepted )
    {
      number = dlg.number->value();
      alone = ( dlg.alone->currentText() == "Yes" );
      QString zs = dlg.zoneset->currentText();
      if( zs.isNull() )
        {
          zs = "";
        }
      if( zs != progRef->getName( zoneset ) )
        {
          if( zoneset != Referencer::null_reference )
            {
              progRef->unbind( zoneset );
            }
          if( zs != "" )
            {
              try
                {
                  zoneset = progRef->getId( zs );
                }
              catch( ... )
                {
                  progRef->bindZoneSet( zoneset );
                  throw;
                }
              progRef->bindZoneSet( zoneset );
            }
        }
    }
}

void ProgActionPlace::setOptionsFrom(const ProgElement* copy)
{
  const ProgActionPlace* o = dynamic_cast<const ProgActionPlace*>( copy );
  if( o != NULL )
    {
      number = o->number;
      alone = o->alone;
      zoneset = o->zoneset;
      if( zoneset != Referencer::null_reference )
        {
          progRef->bind_nofail( zoneset );
        }
    }
}

QDomElement ProgActionPlace::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "place" );
  xml.setAttribute( "number", QString::number( number ) );
  xml.setAttribute( "alone", ( alone ? "yes" : "no" ) );
  addChildToXml( doc, xml );
  progRef->saveZoneSetRef( progRef->getName( zoneset ), xml, doc );
  return ProgAction::getXmlElement( doc, xml );
}

void ProgActionPlace::setFromXml( QDomElement& root )
{
  ProgAction::setFromXml( root );
  sortChildItems( 1, true );
  QDomElement xml = root.firstChild().toElement();
  while( xml.tagName() != "place" )
    {
      xml = xml.nextSibling().toElement();
    }
  number = xml.attribute( "number", "1" ).toInt();
  alone = ( xml.attribute( "alone", "no" ) == "yes" );
  QDomElement child = xml.firstChild().toElement();
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  while( real_child->key( 1, true ) != "2" )
    {
      real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
    }
  if( real_child == NULL )
    {
      cerr << "Error when creating the place object ! " << endl;
      ErrorDlg dlg( "Creation error", "Error when creating the place object ! " );
      dlg.exec();
      return;
    }
  ProgElement* new_child;
  if( ! child.isNull() )
    {
      if( !ProgObjectPlace::isObjectPlaceTag( child.tagName() ) )
        {
          cerr << "'place' element child must be a placeable object" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgObjectPlace::getElement( child, this, progRef );
      if( new_child != NULL )
        {
          real_child = real_child->replaceWith( new_child );
          real_child->transferRefs( new_child );
          delete new_child;
        }
    }
  child = child.nextSibling().toElement();
  if( ! child.isNull() )
    {
      if( child.tagName() != "zone_set" )
        {
          cerr << "'place' element second child must be a zone_set" << endl;
          throw InvalidXmlException();
        }
      zoneset = progRef->xmlZoneSet( child );
      progRef->bind_nofail( zoneset );
    }
}

QPixmap* ProgActionDestroy::icon = NULL;

ProgActionDestroy::ProgActionDestroy(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgAction( parent, pref, index, opt, name )
  , ref( Referencer::null_reference )
  , child_find( NULL )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "action_destroy.png" );
    }
  setPixmap( 0, *icon );
  ProgTest* test = new ProgTest( this, pref, "1", true );
  test->setUniq( true );
  //new ProgFind( this, pref, "2" );
}

ProgActionDestroy::~ProgActionDestroy()
{
  try
    {
      if( ref != Referencer::null_reference )
        {
          progRef->unbind( ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgActionDestroy" << endl;
    }
}

void ProgActionDestroy::transferRefs( ProgElement* source )
{
  ProgActionDestroy* src = dynamic_cast<ProgActionDestroy*>( source );
  if( src != NULL )
    {
      ProgAction::transferRefs( source );
    }
}

ProgElement* ProgActionDestroy::getCopy(ProgElement* parent, QString index) const
{
  return new ProgActionDestroy( parent, progRef, index );
}

void ProgActionDestroy::config( QWidget* parent, const char* name )
{
  DestroyConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos = i+1;
        }
    }
  dlg.ref->setCurrentItem( pos );

  if( dlg.exec() == QDialog::Accepted )
    {
      QString nr = dlg.ref->currentText();
      if( nr == "(None)" )
        {
          nr = "";
        }
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
              /*
              if( child_find == NULL )
                {
                  child_find = firstChild();
                  takeItem( child_find );
                }
                */
            }
          /*
          else
            {
              if( child_find != NULL )
                {
                  insertItem( child_find );
                  child_find = NULL;
                }
            }
            */
        }
    }
}

void ProgActionDestroy::setOptionsFrom(const ProgElement* copy)
{
  const ProgActionDestroy* o = dynamic_cast<const ProgActionDestroy*>( copy );
  if( o != NULL )
    {
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
    }
}

QDomElement ProgActionDestroy::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "destroy" );
  if( ref != Referencer::null_reference )
    {
      QDomElement xml_ref = doc.createElement( "ref" );
      xml_ref.setAttribute( "name", progRef->getName( ref ) );
      xml.appendChild( xml );
    }
    /*
  else
    {
      addChildToXml( doc, xml );
    }
    */
  return ProgAction::getXmlElement( doc, xml );
}

void ProgActionDestroy::setFromXml( QDomElement& root )
{
  ProgAction::setFromXml( root );
  QDomElement xml = root.firstChild().toElement();
  while( xml.tagName() != "destroy" )
    {
      xml = xml.nextSibling().toElement();
    }
  QDomElement child = xml.firstChild().toElement();
  if( ! child.isNull() )
    {
      if( child.tagName() == "ref" )
        {
          ref = progRef->getId_nofail( child.attribute( "name" ) );
          if( ref != Referencer::null_reference )
            {
              progRef->bind_nofail( ref );
            }
        }
      else if( child.tagName() == "find" )
        {
        /*
          ProgElement* find = dynamic_cast<ProgElement*>( firstChild() );
          while( find->key( 1, true ) != "2" )
            {
              find = dynamic_cast<ProgElement*>( find->nextSibling() );
            }
          find->setFromXml( child );
          */
        }
      else
        {
          cerr << "'destroy' element must include 'ref' or 'find'" << endl;
          throw InvalidXmlException();
        }
    }
}

QPixmap* ProgFind::icon = NULL;

ProgFind::ProgFind(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgElement( parent, pref, index, opt, name )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "find.png" );
    }
  setPixmap( 0, *icon );
  new ProgObjectFind( this, pref, "1" );
  new ProgTest( this, pref, "2" );
}

ProgElement* ProgFind::getCopy(ProgElement* parent, QString index) const
{
  return new ProgFind( parent, progRef, index );
}

void ProgFind::transferRefs( ProgElement* source )
{
  ProgFind* src = dynamic_cast<ProgFind*>( source );
  if( src != NULL )
    {
      ProgElement::transferRefs( source );
    }
}

void ProgFind::config( QWidget* parent, const char* name )
{
}

void ProgFind::setOptionsFrom(const ProgElement* copy)
{
}

QDomElement ProgFind::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "find" );
  addChildToXml( doc, xml );
  return xml;
}

void ProgFind::setFromXml( QDomElement& xml )
{
  sortChildItems( 1,true );
  QDomElement child = xml.firstChild().toElement();
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  while( real_child->key( 1, true ) != "1" )
    {
      real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
    }
  ProgElement* new_child;
  ProgElement* par = dynamic_cast<ProgElement*>( parent() );
  if( ! child.isNull() )
    {
      if( ! ProgObjectFind::isObjectFindTag( child.tagName() ) )
        {
          cerr << "'find' element first child must be a findable object" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgObjectFind::getElement( child, par, progRef );
      if( new_child != NULL )
        {
          real_child = real_child->replaceWith( new_child );
          real_child->transferRefs( new_child );
          delete new_child;
        }
    }
  sortChildItems( 1,true );
  real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
  child = child.nextSibling().toElement();
  if( ! child.isNull() )
    {
      if( child.tagName() != "cond" )
        {
          cerr << "'find' element second child must be a cond element" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgTest::getElement( child, par, progRef );
      if( new_child != NULL )
        {
          real_child = real_child->replaceWith( new_child );
          real_child->transferRefs( new_child );
          delete new_child;
        }
    }
}

QPixmap* ProgObjectTerminator::icon = NULL;

ProgObjectTerminator::ProgObjectTerminator(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgObjectInSquad( parent, pref, index, opt, name )
  , ref( Referencer::null_reference )
  , obj_id( Referencer::null_reference )
  , pos( NONE )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "object_terminator.png" );
    }
  setPixmap( 0, *icon );
}

ProgObjectTerminator::~ProgObjectTerminator()
{
  try
    {
      if( ref != Referencer::null_reference )
        {
          progRef->unbind( ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgObjectTerminator" << endl;
    }
}

void ProgObjectTerminator::breakReferences()
{
  if( obj_id != Referencer::null_reference )
    {
      progRef->eraseReference( obj_id );
      obj_id = Referencer::null_reference;
    }
}

ProgElement* ProgObjectTerminator::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectTerminator( parent, progRef, index );
}

void ProgObjectTerminator::config( QWidget* parent, const char* name )
{
  TerminatorConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos_o = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos_o = i+1;
        }
    }
  dlg.ref->setCurrentItem( pos_o );
  switch( pos )
  {
    case LEFT:
      dlg.Left->setOn( true );
      break;
    case RIGHT:
      dlg.Right->setOn( true );
      break;
    case UP:
      dlg.Up->setOn( true );
      break;
    case DOWN:
      dlg.Down->setOn( true );
      break;
    case NONE:
      break;
  }
  dlg.obj_name->setText( progRef->getName( obj_id ) );

  if( dlg.exec() == QDialog::Accepted )
    {
      if( dlg.Up->state() == QButton::On )
        {
          pos = UP;
        }
      else if( dlg.Down->state() == QButton::On )
        {
          pos = DOWN;
        }
      else if( dlg.Right->state() == QButton::On )
        {
          pos = RIGHT;
        }
      else if( dlg.Left->state() == QButton::On )
        {
          pos = LEFT;
        }
      else
        {
          pos = NONE;
        }
      QString nr = dlg.ref->currentText();
      if( nr == "(None)" )
        {
          nr = "";
        }
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      QString nn = dlg.obj_name->text();
      if( nn != progRef->getName( obj_id ) )
        {
          if( nn != "" )
            {
              if( obj_id != Referencer::null_reference )
                {
                  progRef->renameRef( obj_id, nn );
                }
              else
                {
                  obj_id = progRef->refAsObject( nn, this );
                }
            }
          else
            {
              progRef->eraseReference( obj_id );
              obj_id = Referencer::null_reference;
            }
        }

    }
}

void ProgObjectTerminator::setOptionsFrom(const ProgElement* copy)
{
  const ProgObjectTerminator* o = dynamic_cast<const ProgObjectTerminator*>( copy );
  if( o != NULL )
    {
      pos = o->pos;
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
    }
}

void ProgObjectTerminator::transferRefs( ProgElement* source )
{
  ProgObjectTerminator* src = dynamic_cast<ProgObjectTerminator*>( source );
  if( src != NULL )
    {
      obj_id = src->obj_id;
      src->obj_id = Referencer::null_reference;
    }
}

QDomElement ProgObjectTerminator::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "terminator" );
  if( ref != Referencer::null_reference )
    {
      QDomElement xml_ref = doc.createElement( "ref" );
      xml_ref.setAttribute( "name", progRef->getName( ref ) );
      xml.appendChild( xml );
    }
  xml.setAttribute( "direction", dir_name[ pos ] );
  if( obj_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( obj_id ) );
    }
  return xml;
}

void ProgObjectTerminator::setFromXml( QDomElement& xml )
{
  QDomElement child = xml.firstChild().toElement();
  if( ! child.isNull() )
    {
      if( child.tagName() == "ref" )
        {
          ref = progRef->getId_nofail( child.attribute( "name" ) );
          if( ref != Referencer::null_reference )
            {
              progRef->bind_nofail( ref );
            }
        }
      else
        {
          cerr << "'terminator' element can only include 'ref' element" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
        }
    }
  QString obj_name = xml.attribute( "name", "" );
  if( obj_name != "" )
    {
      try
        {
          obj_id = progRef->refAsObject( obj_name, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          obj_id = Referencer::null_reference;
        }
    }
  else
    {
      obj_id = Referencer::null_reference;
    }
  QString dir = xml.attribute( "direction", "none" );
  if( dir == "north" )
    {
      pos = UP;
    }
  else if( dir == "south" )
    {
      pos = DOWN;
    }
  else if( dir == "east" )
    {
      pos = RIGHT;
    }
  else if( dir == "west" )
    {
      pos = LEFT;
    }
  else
    {
      pos = NONE;
    }
}



QPixmap* ProgObjectSergeant::icon = NULL;

ProgObjectSergeant::ProgObjectSergeant(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgObjectInSquad( parent, pref, index, opt, name )
  , ref( Referencer::null_reference )
  , obj_id( Referencer::null_reference )
  , pos( NONE )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "object_sergeant.png" );
    }
  setPixmap( 0, *icon );
}

ProgObjectSergeant::~ProgObjectSergeant()
{
  try
    {
      if( ref != Referencer::null_reference )
        {
          progRef->unbind( ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgObjectSergeant" << endl;
    }
}

void ProgObjectSergeant::breakReferences()
{
  if( obj_id != Referencer::null_reference )
    {
      progRef->eraseReference( obj_id );
      obj_id = Referencer::null_reference;
    }
}

ProgElement* ProgObjectSergeant::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectSergeant( parent, progRef, index );
}

void ProgObjectSergeant::config( QWidget* parent, const char* name )
{
  SergeantConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos_o = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos_o = i+1;
        }
    }
  dlg.ref->setCurrentItem( pos_o );
  switch( pos )
  {
    case LEFT:
      dlg.Left->setOn( true );
      break;
    case RIGHT:
      dlg.Right->setOn( true );
      break;
    case UP:
      dlg.Up->setOn( true );
      break;
    case DOWN:
      dlg.Down->setOn( true );
      break;
    case NONE:
      break;
  }
  dlg.obj_name->setText( progRef->getName( obj_id ) );

  if( dlg.exec() == QDialog::Accepted )
    {
      if( dlg.Up->state() == QButton::On )
        {
          pos = UP;
        }
      else if( dlg.Down->state() == QButton::On )
        {
          pos = DOWN;
        }
      else if( dlg.Right->state() == QButton::On )
        {
          pos = RIGHT;
        }
      else if( dlg.Left->state() == QButton::On )
        {
          pos = LEFT;
        }
      else
        {
          pos = NONE;
        }
      QString nr = dlg.ref->currentText();
      if( nr == "(None)" )
        {
          nr = "";
        }
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      QString nn = dlg.obj_name->text();
      if( nn != progRef->getName( obj_id ) )
        {
          if( nn != "" )
            {
              if( obj_id != Referencer::null_reference )
                {
                  progRef->renameRef( obj_id, nn );
                }
              else
                {
                  obj_id = progRef->refAsObject( nn, this );
                }
            }
          else
            {
              progRef->eraseReference( obj_id );
              obj_id = Referencer::null_reference;
            }
        }

    }
}

void ProgObjectSergeant::setOptionsFrom(const ProgElement* copy)
{
  const ProgObjectSergeant* o = dynamic_cast<const ProgObjectSergeant*>( copy );
  if( o != NULL )
    {
      pos = o->pos;
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
    }
}

void ProgObjectSergeant::transferRefs( ProgElement* source )
{
  ProgObjectSergeant* src = dynamic_cast<ProgObjectSergeant*>( source );
  if( src != NULL )
    {
      obj_id = src->obj_id;
      src->obj_id = Referencer::null_reference;
    }
}

QDomElement ProgObjectSergeant::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "sergeant" );
  if( ref != Referencer::null_reference )
    {
      QDomElement xml_ref = doc.createElement( "ref" );
      xml_ref.setAttribute( "name", progRef->getName( ref ) );
      xml.appendChild( xml );
    }
  xml.setAttribute( "direction", dir_name[ pos ] );
  if( obj_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( obj_id ) );
    }
  return xml;
}

void ProgObjectSergeant::setFromXml( QDomElement& xml )
{
  QDomElement child = xml.firstChild().toElement();
  if( ! child.isNull() )
    {
      if( child.tagName() == "ref" )
        {
          ref = progRef->getId_nofail( child.attribute( "name" ) );
          if( ref != Referencer::null_reference )
            {
              progRef->bind_nofail( ref );
            }
        }
      else
        {
          cerr << "'sergeant' element can only include 'ref' element" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
        }
    }
  QString obj_name = xml.attribute( "name", "" );
  if( obj_name != "" )
    {
      try
        {
          obj_id = progRef->refAsObject( obj_name, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          obj_id = Referencer::null_reference;
        }
    }
  else
    {
      obj_id = Referencer::null_reference;
    }
  QString dir = xml.attribute( "direction", "none" );
  if( dir == "north" )
    {
      pos = UP;
    }
  else if( dir == "south" )
    {
      pos = DOWN;
    }
  else if( dir == "east" )
    {
      pos = RIGHT;
    }
  else if( dir == "west" )
    {
      pos = LEFT;
    }
  else
    {
      pos = NONE;
    }
}



QPixmap* ProgObjectFlamer::icon = NULL;

ProgObjectFlamer::ProgObjectFlamer(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgObjectInSquad( parent, pref, index, opt, name )
  , ref( Referencer::null_reference )
  , obj_id( Referencer::null_reference )
  , pos( NONE )
  , charge( 1 )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "object_flamer.png" );
    }
  setPixmap( 0, *icon );
}

ProgObjectFlamer::~ProgObjectFlamer()
{
  try
    {
      if( ref != Referencer::null_reference )
        {
          progRef->unbind( ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgObjectFlamer" << endl;
    }
}

void ProgObjectFlamer::breakReferences()
{
  if( obj_id != Referencer::null_reference )
    {
      progRef->eraseReference( obj_id );
      obj_id = Referencer::null_reference;
    }
}

ProgElement* ProgObjectFlamer::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectFlamer( parent, progRef, index );
}

void ProgObjectFlamer::config( QWidget* parent, const char* name )
{
  FlamerConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos_o = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos_o = i+1;
        }
    }
  dlg.ref->setCurrentItem( pos_o );
  dlg.charge->setValue( charge );
  switch( pos )
  {
    case LEFT:
      dlg.Left->setOn( true );
      break;
    case RIGHT:
      dlg.Right->setOn( true );
      break;
    case UP:
      dlg.Up->setOn( true );
      break;
    case DOWN:
      dlg.Down->setOn( true );
      break;
    case NONE:
      break;
  }
  dlg.obj_name->setText( progRef->getName( obj_id ) );

  if( dlg.exec() == QDialog::Accepted )
    {
      charge = dlg.charge->value();
      if( dlg.Up->state() == QButton::On )
        {
          pos = UP;
        }
      else if( dlg.Down->state() == QButton::On )
        {
          pos = DOWN;
        }
      else if( dlg.Right->state() == QButton::On )
        {
          pos = RIGHT;
        }
      else if( dlg.Left->state() == QButton::On )
        {
          pos = LEFT;
        }
      else
        {
          pos = NONE;
        }
      QString nr = dlg.ref->currentText();
      if( nr == "(None)" )
        {
          nr = "";
        }
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      QString nn = dlg.obj_name->text();
      if( nn != progRef->getName( obj_id ) )
        {
          if( nn != "" )
            {
              if( obj_id != Referencer::null_reference )
                {
                  progRef->renameRef( obj_id, nn );
                }
              else
                {
                  obj_id = progRef->refAsObject( nn, this );
                }
            }
          else
            {
              progRef->eraseReference( obj_id );
              obj_id = Referencer::null_reference;
            }
        }

    }
}

void ProgObjectFlamer::setOptionsFrom(const ProgElement* copy)
{
  const ProgObjectFlamer* o = dynamic_cast<const ProgObjectFlamer*>( copy );
  if( o != NULL )
    {
      pos = o->pos;
      charge = o->charge;
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
    }
}

void ProgObjectFlamer::transferRefs( ProgElement* source )
{
  ProgObjectFlamer* src = dynamic_cast<ProgObjectFlamer*>( source );
  if( src != NULL )
    {
      obj_id = src->obj_id;
      src->obj_id = Referencer::null_reference;
    }
}

QDomElement ProgObjectFlamer::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "flamer" );
  if( ref != Referencer::null_reference )
    {
      QDomElement xml_ref = doc.createElement( "ref" );
      xml_ref.setAttribute( "name", progRef->getName( ref ) );
      xml.appendChild( xml );
    }
  xml.setAttribute( "direction", dir_name[ pos ] );
  xml.setAttribute( "charger", QString::number( charge ) );
  if( obj_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( obj_id ) );
    }
  return xml;
}

void ProgObjectFlamer::setFromXml( QDomElement& xml )
{
  QDomElement child = xml.firstChild().toElement();
  if( ! child.isNull() )
    {
      if( child.tagName() == "ref" )
        {
          ref = progRef->getId_nofail( child.attribute( "name" ) );
          if( ref != Referencer::null_reference )
            {
              progRef->bind_nofail( ref );
            }
        }
      else
        {
          cerr << "'terminator' element can only include 'ref' element" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
    }
  QString obj_name = xml.attribute( "name", "" );
  if( obj_name != "" )
    {
      try
        {
          obj_id = progRef->refAsObject( obj_name, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          obj_id = Referencer::null_reference;
        }
    }
  else
    {
      obj_id = Referencer::null_reference;
    }
  QString dir = xml.attribute( "direction", "none" );
  charge = xml.attribute( "charger", "1" ).toInt();
  if( dir == "north" )
    {
      pos = UP;
    }
  else if( dir == "south" )
    {
      pos = DOWN;
    }
  else if( dir == "east" )
    {
      pos = RIGHT;
    }
  else if( dir == "west" )
    {
      pos = LEFT;
    }
  else
    {
      pos = NONE;
    }
}



QPixmap* ProgObjectGenestealer::icon = NULL;

ProgObjectGenestealer::ProgObjectGenestealer(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgObjectInSquad( parent, pref, index, opt, name )
  , ref( Referencer::null_reference )
  , obj_id( Referencer::null_reference )
  , pos( NONE )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "object_genestealer.png" );
    }
  setPixmap( 0, *icon );
}

ProgObjectGenestealer::~ProgObjectGenestealer()
{
  try
    {
      if( ref != Referencer::null_reference )
        {
          progRef->unbind( ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgObjectGenestealer" << endl;
    }
}

void ProgObjectGenestealer::breakReferences()
{
  if( obj_id != Referencer::null_reference )
    {
      progRef->eraseReference( obj_id );
      obj_id = Referencer::null_reference;
    }
}

ProgElement* ProgObjectGenestealer::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectGenestealer( parent, progRef, index );
}

void ProgObjectGenestealer::config( QWidget* parent, const char* name )
{
  GenestealerConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos_o = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos_o = i+1;
        }
    }
  dlg.ref->setCurrentItem( pos_o );
  switch( pos )
  {
    case LEFT:
      dlg.Left->setOn( true );
      break;
    case RIGHT:
      dlg.Right->setOn( true );
      break;
    case UP:
      dlg.Up->setOn( true );
      break;
    case DOWN:
      dlg.Down->setOn( true );
      break;
    case NONE:
      break;
  }
  dlg.obj_name->setText( progRef->getName( obj_id ) );

  if( dlg.exec() == QDialog::Accepted )
    {
      if( dlg.Up->state() == QButton::On )
        {
          pos = UP;
        }
      else if( dlg.Down->state() == QButton::On )
        {
          pos = DOWN;
        }
      else if( dlg.Right->state() == QButton::On )
        {
          pos = RIGHT;
        }
      else if( dlg.Left->state() == QButton::On )
        {
          pos = LEFT;
        }
      else
        {
          pos = NONE;
        }
      QString nr = dlg.ref->currentText();
      if( nr == "(None)" )
        {
          nr = "";
        }
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      QString nn = dlg.obj_name->text();
      if( nn != progRef->getName( obj_id ) )
        {
          if( nn != "" )
            {
              if( obj_id != Referencer::null_reference )
                {
                  progRef->renameRef( obj_id, nn );
                }
              else
                {
                  obj_id = progRef->refAsObject( nn, this );
                }
            }
          else
            {
              progRef->eraseReference( obj_id );
              obj_id = Referencer::null_reference;
            }
        }
    }
}

void ProgObjectGenestealer::setOptionsFrom(const ProgElement* copy)
{
  const ProgObjectGenestealer* o = dynamic_cast<const ProgObjectGenestealer*>( copy );
  if( o != NULL )
    {
      pos = o->pos;
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
    }
}

void ProgObjectGenestealer::transferRefs( ProgElement* source )
{
  ProgObjectGenestealer* src = dynamic_cast<ProgObjectGenestealer*>( source );
  if( src != NULL )
    {
      obj_id = src->obj_id;
      src->obj_id = Referencer::null_reference;
    }
}

QDomElement ProgObjectGenestealer::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "genestealer" );
  if( ref != Referencer::null_reference )
    {
      QDomElement xml_ref = doc.createElement( "ref" );
      xml_ref.setAttribute( "name", progRef->getName( ref ) );
      xml.appendChild( xml );
    }
  xml.setAttribute( "direction", dir_name[ pos ] );
  if( obj_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( obj_id ) );
    }
  return xml;
}

void ProgObjectGenestealer::setFromXml( QDomElement& xml )
{
  QDomElement child = xml.firstChild().toElement();
  if( ! child.isNull() )
    {
      if( child.tagName() == "ref" )
        {
          ref = progRef->getId_nofail( child.attribute( "name" ) );
          if( ref != Referencer::null_reference )
            {
              progRef->bind_nofail( ref );
            }
        }
      else
        {
          cerr << "'terminator' element can only include 'ref' element" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
        }
    }
  QString obj_name = xml.attribute( "name", "" );
  if( obj_name != "" )
    {
      try
        {
          obj_id = progRef->refAsObject( obj_name, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          obj_id = Referencer::null_reference;
        }
    }
  else
    {
      obj_id = Referencer::null_reference;
    }
  QString dir = xml.attribute( "direction", "none" );
  if( dir == "north" )
    {
      pos = UP;
    }
  else if( dir == "south" )
    {
      pos = DOWN;
    }
  else if( dir == "east" )
    {
      pos = RIGHT;
    }
  else if( dir == "west" )
    {
      pos = LEFT;
    }
  else
    {
      pos = NONE;
    }
}



QPixmap* ProgObjectBlip::icon = NULL;

ProgObjectBlip::ProgObjectBlip(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgObjectInSquad( parent, pref, index, opt, name )
  , ref( Referencer::null_reference )
  , obj_id( Referencer::null_reference )
  , nb_gen( -1 )
  , wait( true )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "object_blip.png" );
    }
  setPixmap( 0, *icon );
}

ProgObjectBlip::~ProgObjectBlip()
{
  try
    {
      if( ref != Referencer::null_reference )
        {
          progRef->unbind( ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgObjectBlip" << endl;
    }
}

void ProgObjectBlip::breakReferences()
{
  if( obj_id != Referencer::null_reference )
    {
      progRef->eraseReference( obj_id );
      obj_id = Referencer::null_reference;
    }
}

ProgElement* ProgObjectBlip::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectBlip( parent, progRef, index );
}

void ProgObjectBlip::config( QWidget* parent, const char* name )
{
  BlipConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos_o = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos_o = i+1;
        }
    }
  dlg.ref->setCurrentItem( pos_o );
  dlg.obj_name->setText( progRef->getName( obj_id ) );
  dlg.wait->setCurrentItem( wait ? 0 : 1 );
  dlg.size->setCurrentItem( nb_gen + 1 );

  if( dlg.exec() == QDialog::Accepted )
    {
      nb_gen = dlg.size->currentItem() - 1;
      wait = ( dlg.wait->currentText() == "Yes" );
      QString nr = dlg.ref->currentText();
      if( nr == "(None)" )
        {
          nr = "";
        }
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      QString nn = dlg.obj_name->text();
      if( nn != progRef->getName( obj_id ) )
        {
          if( nn != "" )
            {
              if( obj_id != Referencer::null_reference )
                {
                  progRef->renameRef( obj_id, nn );
                }
              else
                {
                  obj_id = progRef->refAsObject( nn, this );
                }
            }
          else
            {
              progRef->eraseReference( obj_id );
              obj_id = Referencer::null_reference;
            }
        }
    }
}

void ProgObjectBlip::setOptionsFrom(const ProgElement* copy)
{
  const ProgObjectBlip* o = dynamic_cast<const ProgObjectBlip*>( copy );
  if( o != NULL )
    {
      nb_gen = o->nb_gen;
      wait = o->wait;
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
    }
}

void ProgObjectBlip::transferRefs( ProgElement* source )
{
  ProgObjectBlip* src = dynamic_cast<ProgObjectBlip*>( source );
  if( src != NULL )
    {
      obj_id = src->obj_id;
      src->obj_id = Referencer::null_reference;
    }
}

QDomElement ProgObjectBlip::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "blip" );
  if( ref != Referencer::null_reference )
    {
      QDomElement xml_ref = doc.createElement( "ref" );
      xml_ref.setAttribute( "name", progRef->getName( ref ) );
      xml.appendChild( xml );
    }
  xml.setAttribute( "wait", ( wait ? "yes" : "no" ) );
  if( nb_gen == -1 )
    {
      xml.setAttribute( "size", "random" );
    }
  else
    {
      xml.setAttribute( "size", QString::number( nb_gen ) );
    }
  if( obj_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( obj_id ) );
    }
  return xml;
}

void ProgObjectBlip::setFromXml( QDomElement& xml )
{
  QDomElement child = xml.firstChild().toElement();
  if( ! child.isNull() )
    {
      if( child.tagName() == "ref" )
        {
          ref = progRef->getId_nofail( child.attribute( "name" ) );
          if( ref != Referencer::null_reference )
            {
              progRef->bind_nofail( ref );
            }
        }
      else
        {
          cerr << "'terminator' element can only include 'ref' element" << endl;
          throw InvalidXmlException();
        }
    }
  QString obj_name = xml.attribute( "name", "" );
  if( obj_name != "" )
    {
      try
        {
          obj_id = progRef->refAsObject( obj_name, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          obj_id = Referencer::null_reference;
        }
    }
  else
    {
      obj_id = Referencer::null_reference;
    }
  wait = ( xml.attribute( "wait", "yes" ) == "yes" );
  QString str_gen = xml.attribute( "size", "random" );
  if( str_gen == "random" )
    {
      nb_gen = -1;
    }
  else
    {
      nb_gen = str_gen.toInt();
    }
}

QPixmap* ProgObjectSquad::icon = NULL;

ProgObjectSquad::ProgObjectSquad(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgObjectPlace( parent, pref, index, opt, name )
  , ref( Referencer::null_reference )
  , obj_id( Referencer::null_reference )
  , pos( NONE )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "object_squad.png" );
    }
  setPixmap( 0, *icon );
  new ProgObjectInSquad( this, pref, "1", true );
}

ProgObjectSquad::~ProgObjectSquad()
{
  try
    {
      if( ref != Referencer::null_reference )
        {
          progRef->unbind( ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgObjectSquad" << endl;
    }
}

void ProgObjectSquad::breakReferences()
{
  if( obj_id != Referencer::null_reference )
    {
      progRef->eraseReference( obj_id );
      obj_id = Referencer::null_reference;
    }
}

ProgElement* ProgObjectSquad::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectSquad( parent, progRef, index );
}

void ProgObjectSquad::config( QWidget* parent, const char* name )
{
  SquadConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  size_t pos_o = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos_o = i+1;
        }
    }
  dlg.ref->setCurrentItem( pos_o );
  dlg.images->setText( images );
  switch( pos )
  {
    case LEFT:
      dlg.Left->setOn( true );
      break;
    case RIGHT:
      dlg.Right->setOn( true );
      break;
    case UP:
      dlg.Up->setOn( true );
      break;
    case DOWN:
      dlg.Down->setOn( true );
      break;
    case NONE:
      break;
  }
  dlg.obj_name->setText( progRef->getName( obj_id ) );

  if( dlg.exec() == QDialog::Accepted )
    {
      images = dlg.images->text();
      if( dlg.Up->state() == QButton::On )
        {
          pos = UP;
          cerr << "pos is now UP" << endl;
        }
      else if( dlg.Down->state() == QButton::On )
        {
          pos = DOWN;
          cerr << "pos is now DOWN" << endl;
        }
      else if( dlg.Right->state() == QButton::On )
        {
          pos = RIGHT;
          cerr << "pos is now RIGHT" << endl;
        }
      else if( dlg.Left->state() == QButton::On )
        {
          pos = LEFT;
          cerr << "pos is now LEFT" << endl;
        }
      else
        {
          pos = NONE;
          cerr << "pos is now NONE" << endl;
        }
      QString nr = dlg.ref->currentText();
      if( nr == "(None)" )
        {
          nr = "";
        }
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      QString nn = dlg.obj_name->text();
      if( nn != progRef->getName( obj_id ) )
        {
          if( nn != "" )
            {
              if( obj_id != Referencer::null_reference )
                {
                  progRef->renameRef( obj_id, nn );
                }
              else
                {
                  obj_id = progRef->refAsObject( nn, this );
                }
            }
          else
            {
              progRef->eraseReference( obj_id );
              obj_id = Referencer::null_reference;
            }
        }

    }
}

void ProgObjectSquad::setOptionsFrom(const ProgElement* copy)
{
  const ProgObjectSquad* o = dynamic_cast<const ProgObjectSquad*>( copy );
  if( o != NULL )
    {
      pos = o->pos;
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
    }
}

void ProgObjectSquad::transferRefs( ProgElement* source )
{
  ProgObjectSquad* src = dynamic_cast<ProgObjectSquad*>( source );
  if( src != NULL )
    {
      obj_id = src->obj_id;
      src->obj_id = Referencer::null_reference;
      ProgElement::transferRefs( source );
    }
}

QDomElement ProgObjectSquad::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "squad" );
  if( ref != Referencer::null_reference )
    {
      QDomElement xml_ref = doc.createElement( "ref" );
      xml_ref.setAttribute( "name", progRef->getName( ref ) );
      xml.appendChild( xml );
    }
  else
    {
      addChildToXml( doc, xml );
    }
  xml.setAttribute( "direction", dir_name[ pos ] );
  if( obj_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( obj_id ) );
    }
  return xml;
}

void ProgObjectSquad::setFromXml( QDomElement& xml )
{
  QDomElement child = xml.firstChild().toElement();
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  ProgElement* new_child;
  ProgElement* par = dynamic_cast<ProgElement*>( parent() );
  if( ! child.isNull() )
    {
      if( child.tagName() == "ref" )
        {
          ref = progRef->getId_nofail( child.attribute( "name" ) );
          if( ref != Referencer::null_reference )
            {
              progRef->bind_nofail( ref );
            }
        }
      else while( ! child.isNull() )
        {
          if( ! ProgObjectInSquad::isObjectSquadTag( child.tagName() ) )
            {
              cerr << "squad element child must be 'in squad' objects or a ref" << endl;
              ErrorXmlSyntax dlg;
              dlg.exec();
            }
          else
            {
              new_child = ProgObjectInSquad::getElement( child, par, progRef );
              if( new_child != NULL )
                {
                  real_child = real_child->replaceWith( new_child );
                  real_child->transferRefs( new_child );
                  delete new_child;
                  sortChildItems( 1,true );
                  real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
                }
              child = child.nextSibling().toElement();
            }
        }
    }
  QString obj_name = xml.attribute( "name", "" );
  if( obj_name != "" )
    {
      try
        {
          obj_id = progRef->refAsObject( obj_name, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          obj_id = Referencer::null_reference;
        }
    }
  else
    {
      obj_id = Referencer::null_reference;
    }
  QString dir = xml.attribute( "direction", "none" );
  if( dir == "north" )
    {
      pos = UP;
    }
  else if( dir == "south" )
    {
      pos = DOWN;
    }
  else if( dir == "east" )
    {
      pos = RIGHT;
    }
  else if( dir == "west" )
    {
      pos = LEFT;
    }
  else
    {
      pos = NONE;
    }
}

QPixmap* ProgObjectObject::icon = NULL;

ProgObjectObject::ProgObjectObject(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgObjectPlace( parent, pref, index, opt, name )
  , type( "" )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "object_object.png" );
    }
  setPixmap( 0, *icon );
}

ProgElement* ProgObjectObject::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectObject( parent, progRef, index );
}

void ProgObjectObject::config( QWidget* parent, const char* name )
{
  ObjectConfig dlg( parent, name, true );
  dlg.type->setText( type );
  if( dlg.exec() == QDialog::Accepted )
    {
      type = dlg.type->text();
    }
}

void ProgObjectObject::setOptionsFrom(const ProgElement* copy)
{
  const ProgObjectObject* o = dynamic_cast<const ProgObjectObject*>( copy );
  if( o != NULL )
    {
      type = o->type;
    }
}

QDomElement ProgObjectObject::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "object" );
  xml.setAttribute( "type", type );
  return xml;
}

void ProgObjectObject::setFromXml( QDomElement& xml )
{
  type = xml.attribute( "type", "" );
}

QPixmap* ProgObjectCat::icon = NULL;

ProgObjectCat::ProgObjectCat(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgObjectPlace( parent, pref, index, opt, name )
  , obj_id( Referencer::null_reference )
  , ref( Referencer::null_reference )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "object_cat.png" );
    }
  setPixmap( 0, *icon );
}

ProgObjectCat::~ProgObjectCat()
{
  try
    {
      if( ref != Referencer::null_reference )
        {
          progRef->unbind( ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgObjectCat" << endl;
    }
}

void ProgObjectCat::breakReferences()
{
  if( obj_id != Referencer::null_reference )
    {
      progRef->eraseReference( obj_id );
      obj_id = Referencer::null_reference;
    }
}

ProgElement* ProgObjectCat::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectCat( parent, progRef, index );
}

void ProgObjectCat::config( QWidget* parent, const char* name )
{
  CATConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  dlg.obj_name->setText( progRef->getName( obj_id ) );
  size_t pos = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos = i+1;
        }
    }
  dlg.ref->setCurrentItem( pos );

  if( dlg.exec() == QDialog::Accepted )
    {
      QString nr = dlg.ref->currentText();
      if( nr == "(None)" )
        {
          nr = "";
        }
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      QString nn = dlg.obj_name->text();
      if( nn != progRef->getName( obj_id ) )
        {
          if( nn != "" )
            {
              if( obj_id != Referencer::null_reference )
                {
                  progRef->renameRef( obj_id, nn );
                }
              else
                {
                  obj_id = progRef->refAsObject( nn, this );
                }
            }
          else
            {
              progRef->eraseReference( obj_id );
              obj_id = Referencer::null_reference;
            }
        }

    }
}

void ProgObjectCat::setOptionsFrom(const ProgElement* copy)
{
  const ProgObjectCat* o = dynamic_cast<const ProgObjectCat*>( copy );
  if( o != NULL )
    {
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
    }
}

void ProgObjectCat::transferRefs( ProgElement* source )
{
  ProgObjectCat* src = dynamic_cast<ProgObjectCat*>( source );
  if( src != NULL )
    {
      obj_id = src->obj_id;
      src->obj_id = Referencer::null_reference;
    }
}

QDomElement ProgObjectCat::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "cat" );
  if( obj_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( obj_id ) );
    }
  if( ref != Referencer::null_reference )
    {
      QDomElement ref_xml = doc.createElement( "ref" );
      ref_xml.setAttribute( "name", progRef->getName( ref ) );
      xml.appendChild( ref_xml );
    }
  return xml;
}

void ProgObjectCat::setFromXml( QDomElement& xml )
{
  QString obj_name = xml.attribute( "name", "" );
  if( obj_name != "" )
    {
      try
        {
          obj_id = progRef->refAsObject( obj_name, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          obj_id = Referencer::null_reference;
        }
    }
  else
    {
      obj_id = Referencer::null_reference;
    }
  QDomElement child = xml.firstChild().toElement();
  if( !child.isNull() )
    {
      if( child.tagName() != "ref" )
        {
          cerr << "Child of 'cat' must be 'ref' or nothing" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
        }
      else
        {
          ref = progRef->getId_nofail( child.attribute( "name", "" ) );
          if( ref != Referencer::null_reference )
            {
              progRef->bind_nofail( ref );
            }
        }
    }
}

QPixmap* ProgObjectExtensible::icon = NULL;

ProgObjectExtensible::ProgObjectExtensible(ProgElement* parent, ProgRef* pref, QString index, bool opt, QString name )
  : ProgObjectPlace( parent, pref, index, opt, name )
  , desc( "" )
  , ref( Referencer::null_reference )
  , obj_id( Referencer::null_reference )
  , empty( false )
  , attackable( false )
  , fireable( false )
  , shootwith( false )
  , opendoorwith( false )
  , shootable( 0 )
  , takeable( 0 )
  , removeable( 0 )
  , transferable( 0 )
{
  if( icon == NULL )
    {
      icon = new QPixmap( getEditorDir() + "object_extensible.png" );
    }
  setPixmap( 0, *icon );
}

ProgObjectExtensible::~ProgObjectExtensible()
{
  try
    {
      if( ref != Referencer::null_reference )
        {
          progRef->unbind( ref );
        }
    }
  catch( ... )
    {
      cerr << "Exception in ~ProgObjectExtensible" << endl;
    }
}

void ProgObjectExtensible::breakReferences()
{
  if( obj_id != Referencer::null_reference )
    {
      progRef->eraseReference( obj_id );
      obj_id = Referencer::null_reference;
    }
}

ProgElement* ProgObjectExtensible::getCopy(ProgElement* parent, QString index) const
{
  return new ProgObjectExtensible( parent, progRef, index );
}

void ProgObjectExtensible::config( QWidget* parent, const char* name )
{
  ExtensibleConfig dlg( parent, name, true );
  vector<Referencer::reference_id> objects = progRef->objRefs();
  dlg.obj_name->setText( progRef->getName( obj_id ) );
  size_t pos = 0;
  for( size_t i = 0 ; i < objects.size() ; ++i )
    {
      dlg.ref->insertItem( progRef->getName( objects[ i ] ) );
      if( objects[ i ] == ref )
        {
          pos = i+1;
        }
    }
  dlg.ref->setCurrentItem( pos );
  dlg.desc->setText( desc );
  dlg.obj_name->setText( progRef->getName( obj_id ) );
  if( graphic == TOXINDUCT )
    {
      dlg.toxinduct->setChecked( true );
    }
  else if( graphic == ARCHIVES )
    {
      dlg.archives->setChecked( true );
    }
  else
    {
      dlg.machinery->setChecked( true );
    }
  dlg.empty->setChecked( empty );
  dlg.attackable->setChecked( attackable );
  dlg.fireable->setChecked( fireable );
  dlg.shootwith->setChecked( shootwith );
  dlg.opendoorwith->setChecked( opendoorwith );
  dlg.shoot_PA->setValue( shootable );
  dlg.take_PA->setValue( takeable );
  dlg.remove_PA->setValue( removeable );
  dlg.transfer_PA->setValue( transferable );
  if( dlg.exec() == QDialog::Accepted )
    {
      empty = dlg.empty->isChecked();
      attackable = dlg.attackable->isChecked();
      fireable = dlg.fireable->isChecked();
      shootwith = dlg.shootwith->isChecked();
      opendoorwith = dlg.opendoorwith->isChecked();
      shootable = dlg.shoot_PA->value();
      takeable = dlg.take_PA->value();
      removeable = dlg.remove_PA->value();
      transferable = dlg.transfer_PA->value();
      if( dlg.toxinduct->isChecked() )
        {
          graphic = TOXINDUCT;
        }
      else if( dlg.archives->isChecked() )
        {
          graphic = ARCHIVES;
        }
      else
        {
          graphic = MACHINERY;
        }

      QString nr = dlg.ref->currentText();
      if( nr == "(None)" )
        {
          nr = "";
        }
      if( nr != progRef->getName( ref ) )
        {
          if( ref != Referencer::null_reference )
            {
              progRef->unbind( ref );
            }
          if( nr != "" )
            {
              try
                {
                  ref = progRef->getId( nr );
                }
              catch( ... )
                {
                  progRef->bindObject( ref );
                  throw;
                }
              progRef->bindObject( ref );
            }
        }
      QString nn = dlg.obj_name->text();
      if( nn != progRef->getName( obj_id ) )
        {
          if( nn != "" )
            {
              if( obj_id != Referencer::null_reference )
                {
                  progRef->renameRef( obj_id, nn );
                }
              else
                {
                  obj_id = progRef->refAsObject( nn, this );
                }
            }
          else
            {
              progRef->eraseReference( obj_id );
              obj_id = Referencer::null_reference;
            }
        }

    }
}

void ProgObjectExtensible::setOptionsFrom(const ProgElement* copy)
{
  const ProgObjectExtensible* o = dynamic_cast<const ProgObjectExtensible*>( copy );
  if( o != NULL )
    {
      ref = o->ref;
      if( ref != Referencer::null_reference )
        {
          progRef->bind_nofail( ref );
        }
      empty = o->empty;
      attackable = o->attackable;
      fireable = o->fireable;
      shootwith = o->shootwith;
      opendoorwith = o->opendoorwith;
      shootable = o->shootable;
      takeable = o->takeable;
      removeable = o->removeable;
      transferable = o->transferable;
    }
}

void ProgObjectExtensible::transferRefs( ProgElement* source )
{
  ProgObjectExtensible* src = dynamic_cast<ProgObjectExtensible*>( source );
  if( src != NULL )
    {
      obj_id = src->obj_id;
      src->obj_id = Referencer::null_reference;
    }
}

QDomElement ProgObjectExtensible::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "extensible" );
  if( obj_id != Referencer::null_reference )
    {
      xml.setAttribute( "name", progRef->getName( obj_id ) );
    }
  if( ref != Referencer::null_reference )
    {
      QDomElement ref_xml = doc.createElement( "ref" );
      ref_xml.setAttribute( "name", progRef->getName( ref ) );
      xml.appendChild( ref_xml );
    }
  xml.setAttribute( "empty", ( empty ? "yes" : "no" ) );
  xml.setAttribute( "attackable", ( attackable ? "yes" : "no" ) );
  xml.setAttribute( "fireable", ( fireable ? "yes" : "no" ) );
  xml.setAttribute( "shootwith", ( shootwith ? "yes" : "no" ) );
  xml.setAttribute( "opendoorwith", ( opendoorwith ? "yes" : "no" ) );
  xml.setAttribute( "shootable", QString::number( shootable ) );
  xml.setAttribute( "takeable", QString::number( takeable ) );
  xml.setAttribute( "removeable", QString::number( removeable ) );
  xml.setAttribute( "transferable", QString::number( transferable ) );
  switch( graphic )
    {
      case TOXINDUCT:
        xml.setAttribute( "graphic", "toxinduct" );
        break;
      case ARCHIVES:
        xml.setAttribute( "graphic", "archives" );
        break;
      case MACHINERY:
        xml.setAttribute( "graphic", "machinery" );
        break;
    }
  return xml;
}

void ProgObjectExtensible::setFromXml( QDomElement& xml )
{
  QString obj_name = xml.attribute( "name", "" );
  if( obj_name != "" )
    {
      try
        {
          obj_id = progRef->refAsObject( obj_name, this );
        }
      catch( NameConflictException ne )
        {
          ErrorNameConflict dlg( ne.name );
          dlg.exec();
          obj_id = Referencer::null_reference;
        }
    }
  else
    {
      obj_id = Referencer::null_reference;
    }
  QDomElement child = xml.firstChild().toElement();
  if( !child.isNull() )
    {
      if( child.tagName() != "ref" )
        {
          cerr << "Child of 'cat' must be 'ref' or nothing" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
        }
      else
        {
          ref = progRef->getId_nofail( child.attribute( "name", "" ) );
          if( ref != Referencer::null_reference )
            {
              progRef->bind_nofail( ref );
            }
        }
    }
  empty = xml.attribute( "empty", "no");
  attackable = xml.attribute( "attackable", "no");
  fireable = xml.attribute( "fireable", "no");
  shootwith = xml.attribute( "shootwith", "no");
  opendoorwith = xml.attribute( "opendoorwith", "no");
  shootable = xml.attribute( "shootable", "0").toInt();
  takeable = xml.attribute( "takeable", "0").toInt();
  removeable = xml.attribute( "removeable", "0").toInt();
  transferable = xml.attribute( "transferable", "0").toInt();
  QString graph = xml.attribute( "graphic", "toxinduct" );
  if( graph == "toxinduct" )
    {
      graphic = TOXINDUCT;
    }
  else if( graph == "archives" )
    {
      graphic = ARCHIVES;
    }
  else
    {
      graphic = MACHINERY;
    }
}

