/***************************************************************************
                          progitems3.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PROGITEMS3_HH
#define PROGITEMS3_HH

#include "progelement.h"
#include "global.h"

#include <list>

class ProgActionPlace : public ProgAction
{
public:
  ProgActionPlace(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Place");
  ~ProgActionPlace();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  int number;
  bool alone;
  Referencer::reference_id zoneset;
};

class ProgActionDestroy : public ProgAction
{
public:
  ProgActionDestroy(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Destroy");
  ~ProgActionDestroy();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id ref;
  QListViewItem *child_find;
};

class ProgFind : public ProgElement
{
public:
  ProgFind(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Find");
  ~ProgFind() {}
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
};

class ProgObjectTerminator : public ProgObjectInSquad
{
public:
  ProgObjectTerminator(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Terminator");
  ~ProgObjectTerminator();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
  void breakReferences();

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id ref, obj_id;
  POSITION pos;

};


class ProgObjectSergeant : public ProgObjectInSquad
{
public:
  ProgObjectSergeant(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Sergeant");
  ~ProgObjectSergeant();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
  void breakReferences();

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id ref, obj_id;
  POSITION pos;

};


class ProgObjectFlamer : public ProgObjectInSquad
{
public:
  ProgObjectFlamer(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Flamer");
  ~ProgObjectFlamer();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
  void breakReferences();

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id ref, obj_id;
  POSITION pos;
  int charge;

};


class ProgObjectGenestealer : public ProgObjectInSquad
{
public:
  ProgObjectGenestealer(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Genestealer");
  ~ProgObjectGenestealer();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
  void breakReferences();

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id ref, obj_id;
  POSITION pos;

};


class ProgObjectBlip : public ProgObjectInSquad
{
public:
  ProgObjectBlip(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Blip");
  ~ProgObjectBlip();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
  void breakReferences();

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id ref, obj_id;
  int nb_gen;
  bool wait;

};

class ProgObjectSquad : public ProgObjectPlace
{
public:
  ProgObjectSquad(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Squad");
  ~ProgObjectSquad();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
  void breakReferences();

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id ref, obj_id;
  POSITION pos;
  QString images;
  std::list<QListViewItem*> childs;

};

class ProgObjectObject : public ProgObjectPlace
{
public:
  ProgObjectObject(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Object");
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  QString type;

};

class ProgObjectCat : public ProgObjectPlace
{
public:
  ProgObjectCat(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "CAT");
  ~ProgObjectCat();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
  void breakReferences();

protected:
  virtual bool validItem() { return true; }
  static QPixmap* icon;
  Referencer::reference_id obj_id;
  Referencer::reference_id ref;
};

class ProgObjectExtensible : public ProgObjectPlace
{
public:
  ProgObjectExtensible(ProgElement* parent, ProgRef* pref, QString index, bool opt = false, QString name = "Extensible object");
  ~ProgObjectExtensible();
  virtual ProgElement* getCopy(ProgElement* parent, QString index) const;
  virtual void config( QWidget* parent = NULL, const char* name = "" );
  virtual void setOptionsFrom(const ProgElement* copy);
  virtual QDomElement getXmlElement( QDomDocument& doc );
  virtual void setFromXml( QDomElement& xml );
  virtual void transferRefs( ProgElement* source );
  void breakReferences();

protected:
  enum GRAPHIC
  {
    TOXINDUCT,
    ARCHIVES,
    MACHINERY
  };

  virtual bool validItem() { return true; }
  static QPixmap* icon;
  QString desc;
  Referencer::reference_id ref, obj_id;
  GRAPHIC graphic;
  bool empty, attackable, fireable, shootwith, opendoorwith;
  int shootable, takeable, removeable, transferable;
};

#endif // PROGITEMS3_HH

