#include "progmenu.h"
#include "progitems.h"
#include "progref.h"
#include "errordlg.h"
#include "global.h"

#include <iostream>

#include <qpixmap.h>
#include <qiconset.h>

using namespace std;

static QPixmap* cfg_img = NULL;
static QPixmap* delete_img = NULL;
static QPixmap* cut_img = NULL;
static QPixmap* copy_img = NULL;
static QPixmap* paste_img = NULL;

ProgMenu::ProgMenu(ProgRef* r, QWidget* parent, const char* name )
  : QPopupMenu( parent, name )
  , current( NULL )
  , ref(r)
{
  if( cfg_img == NULL )
    {
      cfg_img = new QPixmap( getEditorDir() + "validate.png" );
    }
  if( delete_img == NULL )
    {
      delete_img = new QPixmap( getEditorDir() + "delete.png" );
    }
}

void ProgMenu::popup( ProgElement* item, const QPoint& pos, int indexAtPoint )
{
  setContent( item );
  QPopupMenu::popup( pos, indexAtPoint );
}

void ProgMenu::setContent( ProgElement* item )
{
  ProgElement* elmt;
  clear();
  array.clear();
  bool dontDelete = item->isNeeded() || (item->optional() && !item->uniq());
  insertItem( QIconSet( *cfg_img ), "Configure", this, SLOT( configItem() ), 0, -5);
  if(!dontDelete)
    {
      insertItem( QIconSet( *delete_img ), "Delete", this, SLOT( deleteItem() ), 0, -4);
      insertItem( "Cut", this, SLOT( cutItem() ), 0, -2);
    }
  insertItem( "Copy", this, SLOT( copyItem() ), 0, -3);
  if( ref->canPaste( item ) )
    {
      insertItem( "Paste", this, SLOT( pasteItem() ), 0, -1 );
    }
  insertSeparator();
  current = item;
  size_t num_elmt = 0;
  for( int i = 0 ; i < ref->nbProgItems() ; i++ )
    {
      elmt = ref->getElement( i );
      if( item->isSameClass( elmt ) )
        {
          if( elmt->pixmap( 0 ) != NULL )
            {
              insertItem( QIconSet( *elmt->pixmap( 0 ) ), elmt->getText(), num_elmt );
            }
          else
            {
              insertItem( elmt->getText(), num_elmt );
            }
          array.push_back( elmt );
          num_elmt++;
        }
    }
}

void ProgMenu::deleteItem()
{
  try
    {
      current->breakReferences();
    }
  catch( StillRefException se )
    {
      ErrorStillRef dlg( se.name, parentWidget() );
      dlg.exec();
      return;
    }
  catch( InexistingRefException ie )
    {
      ErrorInexistingRef dlg( ie.name, parentWidget() );
      dlg.exec();
      return;
    }

  if( current->uniq() )
    {
      ProgElement* elmt = current->replaceWith( ref->getBase( current ) );
      elmt->setOptional( true );
    }
  else
    {
      delete current;
    }
  current = NULL;
}

void ProgMenu::copyItem()
{
  ref->copyItem( current );
  current = NULL;
}

void ProgMenu::cutItem()
{
  try
    {
      current->breakReferences();
    }
  catch( StillRefException se )
    {
      ErrorStillRef dlg( se.name, parentWidget() );
      dlg.exec();
      return;
    }
  catch( InexistingRefException ie )
    {
      ErrorInexistingRef dlg( ie.name, parentWidget() );
      dlg.exec();
      return;
    }
  ref->copyItem( current );
  if( current->uniq() )
    {
      ProgElement* elmt = current->replaceWith( ref->getBase( current ) );
      elmt->setOptional( true );
    }
  else
    {
      delete current;
    }
  current = NULL;
}

void ProgMenu::pasteItem()
{
  ref->pasteItem( current );
  current = NULL;
}

ProgElement* ProgMenu::itemType( int pos )
{
  if( pos < 0 )
    {
      return NULL;
    }
  return array[ pos ];
}

void ProgMenu::configItem()
{
  try
    {
      current->config();
    }
  catch( NameConflictException ne )
    {
      ErrorNameConflict dlg( ne.name, parentWidget() );
      dlg.exec();
    }
  catch( InexistingRefException ie )
    {
      ErrorInexistingRef dlg( ie.name, parentWidget() );
      dlg.exec();
    }
  catch( StillRefException re )
    {
      ErrorStillRef dlg( re.name, parentWidget() );
      dlg.exec();
    }
}

