/***************************************************************************
                          progref.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "progref.h"

#include "progitems.h"
#include "progitems2.h"
#include "progitems3.h"
#include "spacezones.h"
#include "objectref.h"

#include <qlistview.h>

#include <iostream>
#include <algorithm>

using namespace std;

ProgRef::ProgRef(ReferenceCenter* rc, SpaceZones* z, ObjectRef* oref)
  : Referencer(rc)
  , copy( NULL )
  , zones( z )
  , objRef( oref )
{
  initItems();
  initBase();
}

ProgRef::ProgRef(const ProgRef& pr)
  : Referencer(pr)
  , copy( NULL )
  , zones( pr.zones )
{
  initItems();
  initBase();
}

ProgRef::~ProgRef()
{
}

void ProgRef::eraseReference( Referencer::reference_id ref )
{
  if( ref == Referencer::null_reference )
    return;
  QString name = refCenter->getName( ref );
  if( refCenter->referenced( ref ) )
    {
      throw StillRefException( name );
    }
  ref_type::iterator condref_it = condRef.find( ref );
  ref_type::iterator evalref_it = evalRef.find( ref );
  if( condref_it != condRef.end() )
    {
      condRef.erase( condref_it );
      refCenter->eraseRef( ref );
    }
  else if( evalref_it != evalRef.end() )
    {
      evalRef.erase( evalref_it );
      refCenter->eraseRef( ref );
    }
  else
    {
      objRef->eraseReference( ref );
    }
}

Referencer::reference_id ProgRef::refAsCond( const QString& name, ProgElement* object )
{
  if( refCenter->exists( name ) )
    {
      cerr << "Error, name " << name << " conflict with another object" << endl;
      throw NameConflictException(name);
    }
  Referencer::reference_id ref = refCenter->createRef( name, this );
  condRef[ ref ] = object;
  return ref;
}

Referencer::reference_id ProgRef::refAsEval( const QString& name, ProgElement* object )
{
  if( refCenter->exists( name ) )
    {
      cerr << "Error, name " << name << " conflict with another object" << endl;
      throw NameConflictException(name);
    }
  Referencer::reference_id ref = refCenter->createRef( name, this );
  evalRef[ ref ] = object;
  return ref;
}

Referencer::reference_id ProgRef::refAsObject( const QString& name, ProgElement* object )
{
  return objRef->refAsObject( name, object );
}

void ProgRef::initItems()
{
  nb_ProgItems = 36;
  QListView* view = new QListView();
  dummy_parent = new ProgElement( view, this, "", false, "" );
  progItems = new ProgElement* [nb_ProgItems];
  progItems[ 0 ] = new ProgTestAnd( dummy_parent, this, "" );
  progItems[ 1 ] = new ProgTestCondRef( dummy_parent, this, "" );
  progItems[ 2 ] = new ProgTestEqual( dummy_parent, this, "" );
  progItems[ 3 ] = new ProgTestEval( dummy_parent, this, "" );
  progItems[ 4 ] = new ProgTestFalse( dummy_parent, this, "" );
  progItems[ 5 ] = new ProgTestInf( dummy_parent, this, "" );
  progItems[ 6 ] = new ProgTestNot( dummy_parent, this, "" );
  progItems[ 7 ] = new ProgTestOnboard( dummy_parent, this, "" );
  progItems[ 8 ] = new ProgTestOr( dummy_parent, this, "" );
  progItems[ 9 ] = new ProgTestPosition( dummy_parent, this, "" );
  progItems[ 10 ] = new ProgTestState( dummy_parent, this, "" );
  progItems[ 11 ] = new ProgTestSup( dummy_parent, this, "" );
  progItems[ 12 ] = new ProgTestTrue( dummy_parent, this, "" );
  progItems[ 13 ] = new ProgTestTurntype( dummy_parent, this, "" );
  progItems[ 14 ] = new ProgValueAP( dummy_parent, this, "" );
  progItems[ 15 ] = new ProgValueBlockedentry( dummy_parent, this, "" );
  progItems[ 16 ] = new ProgValueDead( dummy_parent, this, "" );
  progItems[ 17 ] = new ProgValueLeaved( dummy_parent, this, "" );
  progItems[ 18 ] = new ProgValueProduct( dummy_parent, this, "" );
  progItems[ 19 ] = new ProgValueRandom( dummy_parent, this, "" );
  progItems[ 20 ] = new ProgValueSize( dummy_parent, this, "" );
  progItems[ 21 ] = new ProgValueSum( dummy_parent, this, "" );
  progItems[ 22 ] = new ProgValueTaken( dummy_parent, this, "" );
  progItems[ 23 ] = new ProgValueTurn( dummy_parent, this, "" );
  progItems[ 24 ] = new ProgValueValue( dummy_parent, this, "" );
  progItems[ 25 ] = new ProgActionPlace( dummy_parent, this, "" );
  progItems[ 26 ] = new ProgActionDestroy( dummy_parent, this, "" );
  progItems[ 27 ] = new ProgObjectBlip( dummy_parent, this, "" );
  progItems[ 28 ] = new ProgObjectCat( dummy_parent, this, "" );
  progItems[ 29 ] = new ProgObjectExtensible( dummy_parent, this, "" );
  progItems[ 30 ] = new ProgObjectFlamer( dummy_parent, this, "" );
  progItems[ 31 ] = new ProgObjectGenestealer( dummy_parent, this, "" );
  progItems[ 32 ] = new ProgObjectObject( dummy_parent, this, "" );
  progItems[ 33 ] = new ProgObjectSergeant( dummy_parent, this, "" );
  progItems[ 34 ] = new ProgObjectSquad( dummy_parent, this, "" );
  progItems[ 35 ] = new ProgObjectTerminator( dummy_parent, this, "" );
}

void ProgRef::initBase()
{
  nb_bases = 6;
  progbase = new  ProgElement* [ nb_bases ];
  progbase[0] = new ProgTest( dummy_parent, this, "" );
  progbase[1] = new ProgAction( dummy_parent, this, "" );
  progbase[2] = new ProgValue( dummy_parent, this, "" );
  progbase[3] = new ProgObjectSquad( dummy_parent, this, "" );
  progbase[4] = new ProgObjectPlace( dummy_parent, this, "" );
  progbase[5] = new ProgObjectFind( dummy_parent, this, "" );
}

void ProgRef::copyItem(ProgElement *item)
{
  if( copy != NULL )
    {
      delete copy;
    }
  copy = item->getExactCopy( dummy_parent, "aa");
}

void ProgRef::pasteItem(ProgElement *replaced)
{
  replaced->replaceWith( copy );
}

bool ProgRef::canPaste( const ProgElement *replaced )
{
  if( copy == NULL)
    {
      return false;
    }
  return replaced->isSameClass( copy );
}

ProgElement* ProgRef::getBase( ProgElement *item )
{
  for( int i = 0 ; i < nb_bases ; i++ )
    {
      if( progbase[ i ]->isSameClass( item ) )
        {
          return progbase[ i ];
        }
    }
  return NULL;
}

void ProgRef::bindCond( Referencer::reference_id ref )
{
  if( ref == Referencer::null_reference )
    return;
  ref_type::iterator found = condRef.find( ref );
  if( found == condRef.end() )
    throw InexistingRefException( ref );
  refCenter->bind( ref );
}

void ProgRef::bindEval( Referencer::reference_id ref )
{
  if( ref == Referencer::null_reference )
    return;
  ref_type::iterator found = evalRef.find( ref );
  if( found == evalRef.end() )
    throw InexistingRefException( ref );
  refCenter->bind( ref );
}

void ProgRef::bindObject( Referencer::reference_id ref )
{
  objRef->bindObject( ref );
}

vector<Referencer::reference_id> ProgRef::evalRefs() const
{
  vector<Referencer::reference_id> refs( evalRef.size() );
  ref_type::const_iterator it;
  int i = 0;
  for( it = evalRef.begin() ; it != evalRef.end() ; ++it )
    {
      refs[ i++ ] = it->first;
    }
  sort( refs.begin(), refs.end(), CmpRefsByName( refCenter ) );
  return refs;
}

vector<Referencer::reference_id> ProgRef::objRefs() const
{
  return objRef->objRefs();
}

vector<Referencer::reference_id> ProgRef::condRefs() const
{
  vector<Referencer::reference_id> refs( condRef.size() );
  ref_type::const_iterator it;
  int i = 0;
  for( it = condRef.begin() ; it != condRef.end() ; ++it )
    {
      refs[ i++ ] = it->first;
    }
  sort( refs.begin(), refs.end(), CmpRefsByName( refCenter ) );
  return refs;
}

void ProgRef::unbind( Referencer::reference_id ref )
{
  if( ref == Referencer::null_reference )
    return;
  refCenter->unbind( ref );
}


void ProgRef::bindZone( Referencer::reference_id ref )
{
  refCenter->bind( ref );
}

void ProgRef::bindZoneSet( Referencer::reference_id ref )
{
  refCenter->bind( ref );
}

vector<Referencer::reference_id> ProgRef::zoneRefs() const
{
  vector<Referencer::reference_id> result;
  SpaceZones::zones_type::const_iterator it;
  for( it = zones->zones.begin() ; it != zones->zones.end() ; ++it )
    {
      result.push_back( it->first );
    }
  sort( result.begin(), result.end(), CmpRefsByName( refCenter ) );
  return result;
}

vector<Referencer::reference_id> ProgRef::zonesetRefs() const
{
  vector<Referencer::reference_id> result;
  SpaceZones::zone_sets_type::const_iterator it;
  for( it = zones->zone_sets.begin() ; it != zones->zone_sets.end() ; ++it )
    {
      result.push_back( it->first );
    }
  sort( result.begin(), result.end(), CmpRefsByName( refCenter ) );
  return result;
}

Referencer::reference_id ProgRef::xmlZone( const QDomElement& xml )
{
  return zones->setZone( xml );
}

Referencer::reference_id ProgRef::xmlZoneSet( const QDomElement& xml )
{
  return zones->setZoneSet( xml );
}

void ProgRef::saveZoneRef( const QString& name, QDomElement& root, QDomDocument& doc )
{
  saveRef( "zone", root, doc )( name );
}

void ProgRef::saveZoneSetRef( const QString& name, QDomElement& root, QDomDocument& doc )
{
  saveRef( "zone_set", root, doc )( name );
}

void ProgRef::bind_nofail( Referencer::reference_id ref )
{
  refCenter->bind_nofail( ref );
}

