/***************************************************************************
                          progref.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _PROGREF_HH_
#define _PROGREF_HH_

#include "spaceref.h"
#include "progelement.h"

#include <vector>
#include <map>

using namespace std;

class SpaceZones;
class ObjectRef;

class ProgRef : public Referencer
{
public:
  typedef map<Referencer::reference_id, ProgElement*> ref_type;
  ProgRef(ReferenceCenter* rc, SpaceZones* z, ObjectRef* oref);
  ProgRef(const ProgRef& pr);
  ~ProgRef();

  void eraseReference ( Referencer::reference_id ref );
  Referencer::reference_id refAsCond( const QString& name, ProgElement* elmt );
  Referencer::reference_id refAsEval( const QString& name, ProgElement* elmt );
  Referencer::reference_id refAsObject( const QString& name, ProgElement* elmt );

  int nbProgItems() const { return nb_ProgItems; }
  ProgElement* getElement(int i) { return progItems[i]; }
  ProgElement* getBase(ProgElement *item);
  void unbind( Referencer::reference_id ref );

  void bindCond( Referencer::reference_id ref );
  void bindEval( Referencer::reference_id ref );
  void bindZone( Referencer::reference_id ref );
  void bindZoneSet( Referencer::reference_id ref );
  void bindObject( Referencer::reference_id ref );

  void bind_nofail( Referencer::reference_id ref );

  Referencer::reference_id xmlZoneSet( const QDomElement& xml );
  void saveZoneSetRef( const QString& name, QDomElement& root, QDomDocument& doc );
  Referencer::reference_id xmlZone( const QDomElement& xml );
  void saveZoneRef( const QString& name, QDomElement& root, QDomDocument& doc );

  void copyItem( ProgElement *item );
  void pasteItem( ProgElement *replaced );
  bool canPaste( const ProgElement *replaced );

  vector<Referencer::reference_id> condRefs() const;
  vector<Referencer::reference_id> evalRefs() const;
  vector<Referencer::reference_id> zoneRefs() const;
  vector<Referencer::reference_id> zonesetRefs() const;
  vector<Referencer::reference_id> objRefs() const;

protected:
  void initItems();
  void initBase();

  ProgElement *copy;
  ProgElement *dummy_parent;
  ProgElement **progItems;
  ProgElement **progbase;
  int nb_ProgItems, nb_bases;
  SpaceZones* zones;
  ObjectRef *objRef;

  ref_type condRef;
  ref_type evalRef;
  ref_type objectRef;

};

#endif
