/***************************************************************************
                          progtree.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "progtree.h"
#include "progwnd.h"
#include "global.h"
#include "errordlg.h"

#include <qpixmap.h>
#include <iostream>

using namespace std;

QPixmap *ProgTree::bitmap = NULL;

ProgTree::ProgTree(QListView* p, ProgRef *pref, QString index, const QString& name)
  : ProgElement(p, pref, index, false, name )
{
  if(bitmap == NULL)
    {
      bitmap = new QPixmap("tree.png");
    }
  setPixmap( 0, *bitmap );
  isUniq = true;
}

ProgTree::~ProgTree()
{
}

bool ProgTree::isSameClass(ProgElement* element) const
{
  ProgTree* convert = dynamic_cast<ProgTree*>(element);
  return convert != NULL;
}

ProgTreeGlobalInit::ProgTreeGlobalInit(QListView* parent, ProgRef* pref, QString index )
  : ProgTree( parent, pref, index, "Global initialisation" )
{
  new ProgAction( this, pref, "1", true );
}

QDomElement ProgTreeGlobalInit::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "init" );
  addChildToXml( doc, xml );
  if( xml.childNodes().count() == 0 )
    {
      return QDomElement();
    }
  return xml;
}

void ProgTreeGlobalInit::setFromXml( QDomElement& xml )
{
  QListView dummy;
  ProgElement dummy_parent( &dummy, progRef, "", false, "" );
  QDomElement child = xml.firstChild().toElement();
  sortChildItems( 1,true );
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( child.tagName() != "action" )
        {
          cerr << "'init' element child must be an action" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
        }
      else
        {
          new_child = ProgAction::getElement( child, &dummy_parent, progRef );
          if( new_child != NULL )
            {
              real_child = real_child->replaceWith( new_child );
              real_child->transferRefs( new_child );
              delete new_child;
              sortChildItems( 1,true );
              real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
            }
        }
      child = child.nextSibling().toElement();
    }
}

ProgTreeDraw::ProgTreeDraw(QListView* parent, ProgRef* pref, QString index )
  : ProgTree( parent, pref, index, "Draw conditions" )
{
  ProgTest *t = new ProgTest( this, pref, "1", true );
  t->setUniq();
}

QDomElement ProgTreeDraw::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "draw" );
  addChildToXml( doc, xml );
  if( xml.childNodes().count() == 0 )
    {
      return QDomElement();
    }
  return xml;
}

void ProgTreeDraw::setFromXml( QDomElement& xml )
{
  QListView dummy;
  ProgElement dummy_parent( &dummy, progRef, "", false, "" );
  QDomElement child = xml.firstChild().toElement();
  if( child.tagName() != "cond" )
    {
      cerr << "Error, the child of 'win' should be 'cond'" << endl;
      ErrorXmlSyntax dlg;
      dlg.exec();
      return;
    }
  ProgTest* test = ProgTest::getElement( child, &dummy_parent, progRef );
  if( test != NULL )
    {
      ProgElement* elmt = dynamic_cast<ProgElement*>( firstChild () );
      elmt = elmt->replaceWith( test );
      elmt->transferRefs( test );
      delete test;
    }
}

ProgTreeMarineBegin::ProgTreeMarineBegin(QListView* parent, ProgRef* pref, QString index )
  : ProgTree( parent, pref, index, "Marine begin of turn" )
{
  new ProgAction( this, pref, "1", true );
}

QDomElement ProgTreeMarineBegin::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "begin" );
  addChildToXml( doc, xml );
  if( xml.childNodes().count() == 0 )
    {
      return QDomElement();
    }
  return xml;
}

void ProgTreeMarineBegin::setFromXml( QDomElement& xml )
{
  QListView dummy;
  ProgElement dummy_parent( &dummy, progRef, "", false, "" );
  QDomElement child = xml.firstChild().toElement();
  sortChildItems( 1,true );
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( child.tagName() != "action" )
        {
          cerr << "'init' element child must be an action" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgAction::getElement( child, &dummy_parent, progRef );
      if( new_child != NULL )
        {
          real_child = real_child->replaceWith( new_child );
          real_child->transferRefs( new_child );
          delete new_child;
          sortChildItems( 1,true );
          real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
        }
      child = child.nextSibling().toElement();
    }
}

ProgTreeMarineInit::ProgTreeMarineInit(QListView* parent, ProgRef* pref, QString index )
  : ProgTree( parent, pref, index, "Marine initialisation" )
{
  new ProgAction( this, pref, "1", true );
}

QDomElement ProgTreeMarineInit::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "init" );
  addChildToXml( doc, xml );
  if( xml.childNodes().count() == 0 )
    {
      return QDomElement();
    }
  return xml;
}

void ProgTreeMarineInit::setFromXml( QDomElement& xml )
{
  QListView dummy;
  ProgElement dummy_parent( &dummy, progRef, "", false, "" );
  QDomElement child = xml.firstChild().toElement();
  sortChildItems( 1,true );
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( child.tagName() != "action" )
        {
          cerr << "'init' element child must be an action" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgAction::getElement( child, &dummy_parent, progRef );
      if( new_child != NULL )
        {
          real_child = real_child->replaceWith( new_child );
          real_child->transferRefs( new_child );
          delete new_child;
          sortChildItems( 1,true );
          real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
        }
      child = child.nextSibling().toElement();
    }
}

ProgTreeMarineWin::ProgTreeMarineWin(QListView* parent, ProgRef* pref, QString index )
  : ProgTree( parent, pref, index, "Marine wining conditions" )
{
  new ProgTest( this, pref, "1" );
}

QDomElement ProgTreeMarineWin::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "win" );
  addChildToXml( doc, xml );
  return xml;
}

void ProgTreeMarineWin::setFromXml( QDomElement& xml )
{
  QListView dummy;
  ProgElement dummy_parent( &dummy, progRef, "", false, "" );
  QDomElement child = xml.firstChild().toElement();
  if( child.tagName() != "cond" )
    {
      cerr << "Error, the child of 'win' should be 'cond'" << endl;
      ErrorXmlSyntax dlg;
      dlg.exec();
      return;
    }
  ProgTest* test = ProgTest::getElement( child, &dummy_parent, progRef );
  if( test != NULL )
    {
      ProgElement* elmt = dynamic_cast<ProgElement*>( firstChild () );
      elmt = elmt->replaceWith( test );
      elmt->transferRefs( test );
      delete test;
    }
}

ProgTreeGenestealerBegin::ProgTreeGenestealerBegin(QListView* parent, ProgRef* pref, QString index )
  : ProgTree( parent, pref, index, "Genestealer begin of turn" )
{
  new ProgAction( this, pref, "1", true );
}

QDomElement ProgTreeGenestealerBegin::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "begin" );
  addChildToXml( doc, xml );
  if( xml.childNodes().count() == 0 )
    {
      return QDomElement();
    }
  return xml;
}

void ProgTreeGenestealerBegin::setFromXml( QDomElement& xml )
{
  QListView dummy;
  ProgElement dummy_parent( &dummy, progRef, "", false, "" );
  QDomElement child = xml.firstChild().toElement();
  sortChildItems( 1,true );
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( child.tagName() != "action" )
        {
          cerr << "'init' element child must be an action" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgAction::getElement( child, &dummy_parent, progRef );
      if( new_child != NULL )
        {
          real_child = real_child->replaceWith( new_child );
          real_child->transferRefs( new_child );
          delete new_child;
          sortChildItems( 1,true );
          real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
        }
      else
        {
          cerr << "Unhandled action " << child.firstChild().toElement().tagName() << endl;
        }
      child = child.nextSibling().toElement();
    }
}

ProgTreeGenestealerInit::ProgTreeGenestealerInit(QListView* parent, ProgRef* pref, QString index )
  : ProgTree( parent, pref, index, "Genestealer initialisation" )
{
  new ProgAction( this, pref, "1", true );
}

QDomElement ProgTreeGenestealerInit::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "init" );
  addChildToXml( doc, xml );
  if( xml.childNodes().count() == 0 )
    {
      return QDomElement();
    }
  return xml;
}

void ProgTreeGenestealerInit::setFromXml( QDomElement& xml )
{
  QListView dummy;
  ProgElement dummy_parent( &dummy, progRef, "", false, "" );
  QDomElement child = xml.firstChild().toElement();
  sortChildItems( 1,true );
  ProgElement* real_child = dynamic_cast<ProgElement*>( firstChild() );
  ProgElement* new_child;
  while( ! child.isNull() )
    {
      if( child.tagName() != "action" )
        {
          cerr << "'init' element child must be an action" << endl;
          ErrorXmlSyntax dlg;
          dlg.exec();
          return;
        }
      new_child = ProgAction::getElement( child, &dummy_parent, progRef );
      if( new_child != NULL )
        {
          real_child = real_child->replaceWith( new_child );
          real_child->transferRefs( new_child );
          delete new_child;
          sortChildItems( 1,true );
          real_child = dynamic_cast<ProgElement*>( real_child->nextSibling() );
        }
      child = child.nextSibling().toElement();
    }
}

ProgTreeGenestealerWin::ProgTreeGenestealerWin(QListView* parent, ProgRef* pref, QString index )
  : ProgTree( parent, pref, index, "Genestealer wining conditions" )
{
  new ProgTest( this, pref, "1" );
}

QDomElement ProgTreeGenestealerWin::getXmlElement( QDomDocument& doc )
{
  QDomElement xml = doc.createElement( "win" );
  addChildToXml( doc, xml );
  return xml;
}

void ProgTreeGenestealerWin::setFromXml( QDomElement& xml )
{
  QListView dummy;
  ProgElement dummy_parent( &dummy, progRef, "", false, "" );
  QDomElement child = xml.firstChild().toElement();
  if( child.tagName() != "cond" )
    {
      cerr << "Error, the child of 'win' should be 'cond'" << endl;
      ErrorXmlSyntax dlg;
      dlg.exec();
      return;
    }
  ProgTest* test = ProgTest::getElement( child, &dummy_parent, progRef );
  if( test != NULL )
    {
      ProgElement* elmt = dynamic_cast<ProgElement*>( firstChild () );
      elmt = elmt->replaceWith( test );
      elmt->transferRefs( test );
      delete test;
    }
}

