/***************************************************************************
                          progwnd.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "progwnd.h"
#include "progtree.h"
#include "progmenu.h"
#include "progitems.h"

#include "progelement.h"
#include "progref.h"
#include "errordlg.h"

#include <qpainter.h>
#include <qevent.h>
#include <qlistview.h>
#include <qdom.h>

#include <iostream>

using namespace std;

ProgWnd::ProgWnd( QWidget * parent, const char * name, WFlags f)
  : QVBox(parent, name, f)
  , current( NULL )
  , menu( NULL )
  , ref( NULL )
  , refCenter( NULL )
  , smap( NULL )
  , zones( NULL )
  , objRef( NULL )
{
  //view = new QScrollView(this);
  view = new QListView(this);
  view->addColumn( "Programm tree" );

  connect( view, SIGNAL( rightButtonClicked ( QListViewItem *, const QPoint &, int ) ),
	   this, SLOT( changeItem( QListViewItem *, const QPoint &, int ) ) );
  connect( view, SIGNAL( doubleClicked ( QListViewItem * ) ),
           this, SLOT( configItem( QListViewItem * ) ) );
}

ProgWnd::~ProgWnd()
{
}

void ProgWnd::createTrees()
{
  //view->addChild(vb);
  trees[ 0 ] = new ProgTreeGlobalInit(view, ref, "1");
  trees[ 1 ] = new ProgTreeDraw(view, ref, "2" );
  trees[ 2 ] = new ProgTreeMarineInit(view, ref, "3" );
  trees[ 3 ] = new ProgTreeMarineBegin(view, ref, "4" );
  trees[ 4 ] = new ProgTreeMarineWin(view, ref, "5" );
  trees[ 5 ] = new ProgTreeGenestealerInit(view, ref, "6" );
  trees[ 6 ] = new ProgTreeGenestealerBegin(view,ref,  "7" );
  trees[ 7 ] = new ProgTreeGenestealerWin(view, ref, "8" );
  view->sort();
}

void ProgWnd::changeItem( QListViewItem *item, const QPoint &pt, int )
{
  ProgTree *tree = dynamic_cast<ProgTree*>( item );
  if( tree != NULL )
    {
      return;
    }
  ProgElement *elmt = dynamic_cast<ProgElement*>( item );
  if( elmt == NULL )
    {
      return;
    }
  current = elmt;
  if( menu == NULL )
    {
      cerr << "Error, must call ProgWnd::setRef first !" << endl;
      exit( 1 );
    }
  menu->popup( elmt, pt );
}

void ProgWnd::changeItem( int pos)
{
  if( current != NULL )
    {
      ProgElement *elmt = menu->itemType( pos );
      if( elmt != NULL )
	{
	  try
	    {
	      current->breakReferences();
	      ProgElement *intable = current->replaceWith( elmt );
	      intable->config( this, "config_replaced" );
	    }
	  catch( StillRefException se )
	    {
	      ErrorStillRef dlg( se.name, parentWidget() );
	      dlg.exec();
	      return;
	    }
	  catch( InexistingRefException ie )
	    {
	      ErrorInexistingRef dlg( ie.name, parentWidget() );
	      dlg.exec();
	      return;
	    }
          catch( NameConflictException ne )
            {
              ErrorNameConflict dlg( ne.name, parentWidget() );
              dlg.exec();
              return;
            }
	}
      current = NULL;
    }
  else
    {
      cerr << "Strange : current is NULL !! " << endl;
    }
}

void ProgWnd::setRef(SpaceMap* m, SpaceZones* z, ReferenceCenter *rc, ObjectRef* oref)
{
  smap = m;
  zones = z;
  refCenter = rc;
  objRef = oref;
  ref = new ProgRef( refCenter, z, oref );
  menu = new ProgMenu( ref, this, "item popup" );
  connect( menu, SIGNAL( activated( int ) ), this, SLOT( changeItem( int ) ) );
  createTrees();
}

void ProgWnd::configItem( QListViewItem* item )
{
  ProgElement* elmt = dynamic_cast<ProgElement*>( item );
  elmt->config();
}

QDomElement ProgWnd::getXmlForGlobal( QDomDocument& doc ) const
{
  return trees[ 0 ]->getXmlElement( doc );
}

QDomElement ProgWnd::getXmlForDraw( QDomDocument& doc ) const
{
  return trees[ 1 ]->getXmlElement( doc );
}

QDomElement ProgWnd::getXmlForMarineInit( QDomDocument& doc ) const
{
  return trees[ 2 ]->getXmlElement( doc );
}

QDomElement ProgWnd::getXmlForMarineBegin( QDomDocument& doc ) const
{
  return trees[ 3 ]->getXmlElement( doc );
}

QDomElement ProgWnd::getXmlForMarineWin( QDomDocument& doc ) const
{
  return trees[ 4 ]->getXmlElement( doc );
}

QDomElement ProgWnd::getXmlForGenestealerInit( QDomDocument& doc ) const
{
  return trees[ 5 ]->getXmlElement( doc );
}

QDomElement ProgWnd::getXmlForGenestealerBegin( QDomDocument& doc ) const
{
  return trees[ 6 ]->getXmlElement( doc );
}

QDomElement ProgWnd::getXmlForGenestealerWin( QDomDocument& doc ) const
{
  return trees[ 7 ]->getXmlElement( doc );
}

void ProgWnd::setGlobalFromXml( QDomElement& xml )
{
  trees[ 0 ]->setFromXml( xml );
}

void ProgWnd::setDrawFromXml( QDomElement& xml )
{
  trees[ 1 ]->setFromXml( xml );
}

void ProgWnd::setMarineInitFromXml( QDomElement& xml )
{
  trees[ 2 ]->setFromXml( xml );
}

void ProgWnd::setMarineBeginFromXml( QDomElement& xml )
{
  trees[ 3 ]->setFromXml( xml );
}

void ProgWnd::setMarineWinFromXml( QDomElement& xml )
{
  trees[ 4 ]->setFromXml( xml );
}

void ProgWnd::setGenestealerInitFromXml( QDomElement& xml )
{
  trees[ 5 ]->setFromXml( xml );
}

void ProgWnd::setGenestealerBeginFromXml( QDomElement& xml )
{
  trees[ 6 ]->setFromXml( xml );
}

void ProgWnd::setGenestealerWinFromXml( QDomElement& xml )
{
  trees[ 7 ]->setFromXml( xml );
}

