/***************************************************************************
                          spacebars.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SPACEBARS_HH_
#define _SPACEBARS_HH_

#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qbuttongroup.h>

#include <list>

#include "config.h"
#include "rcobject.h"
#include "spacetools.h"
#include "spacemap.h"
#include "spacezones.h"

using namespace std;

class QLabel;
class QComboBox;
class QToolButton;

class SpaceMapBar;

class SpaceButton : public QToolButton
{
  Q_OBJECT
public:
  SpaceButton(SpaceMapBar* b, SpaceToolPtr t, QToolBar * parent, const char * name = 0);

public slots:
  void setOn ( bool enable );
  void toggle ();

protected:
  void action();
  void idle();

  bool pressed;
  SpaceToolPtr tool;
  SpaceMapBar* bar;
};

class SpaceMapBar : public QToolBar
{
  Q_OBJECT
public:
  SpaceMap* map;
  SpaceMapBar(const QString & label, QMainWindow *wnd);
  virtual ~SpaceMapBar();
  void install(SpaceToolPtr tool);
  void setMap(SpaceMap* m);
  void changeButton(SpaceButton* button);
  void makeIdle(const SpaceButton *button);

signals:
  void changeTool(const QString& toolname);

protected:
  SpaceButton *last;
};

class SpaceZonesBar : public QToolBar
{
  Q_OBJECT
public:
  SpaceZonesBar(const QString & label, QMainWindow *wnd);
  virtual ~SpaceZonesBar();
  void setZones(SpaceZones* z, SpaceMap* m);
  void reload();

public slots:
  void setZone();
  void renameZone();
  void deleteZone();
  void editZoneSet();
  void deleteZoneSet();
  void showZone(const QString& name);
  void showZoneSet(const QString& name);
  void unshowZoneSel();

protected:
  void reloadZones();
  void reloadSets();

  SpaceZones *zones;
  SpaceMap *map;
  QToolButton *unshow_sel;
  QLabel *zones_label;
  QLabel *set_label;
  QComboBox *cb_zones;
  QToolButton *validate;
  QToolButton *rename_zone;
  QToolButton *erase_zone;
  QComboBox *cb_zonesets;
  QToolButton *edit_set;
  QToolButton *erase_set;
};

#endif
