/***************************************************************************
                          spaceditor.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SPACEDITOR_HH_
#define _SPACEDITOR_HH_

#include "config.h"

#include <qapplication.h>
#include <qmainwindow.h>

#include "spacemap.h"

class SpaceMapBar;
class SpaceZonesBar;
class SpaceZones;
class ReferenceCenter;
class SpaceDescription;
class ProgEditor;
class ObjectRef;

class QStatusBar;
class QPopupMenu;
class QScrollView;
class QMenuBar;
class QLabel;

class SpacEditor : public QApplication
{
  Q_OBJECT
public:
  SpacEditor(int argc, char *argv[]);
  ~SpacEditor();

public slots:
  void newFile();
  void openFile();
  void closeFile();
  void saveFile();
  void quit();
  void setTool(const QString& toolname);
  void setFile(const QString& filename);
  void editDesc();

  void selectAllEdit();
  void inverseSelectEdit();
  void unselectEdit();

  void setCheck(int itemId);
  void changeSize();
protected:

  void createMenu();
  void createStatusBar();
  void createObjects(int w, int h);
  void resetObjects(int w, int h);
  void deleteObjects();

  void RegisterTools();

  QMainWindow main_wnd;
  QMenuBar *menu;
  QStatusBar *status;

  QPopupMenu *fileMenu, *editMenu, *helpMenu, *viewMenu;
  int changeSizeId;

  SpaceMap* smap;
  SpaceZones* zones;
  ReferenceCenter* refCenter;
  SpaceDescription* desc;
  ProgEditor *prog;
  ObjectRef *objRef;

  QWidget* mainBox;
  QScrollView* mainView;
  SpaceMapBar* mapBar;
  SpaceZonesBar* zonesBar;
  QStatusBar* statusBar;
  QLabel *fileLoaded;
  QLabel *currentTool;
};

#endif
