/***************************************************************************
                          spacemap.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SPACEMAP_HH_
#define _SPACEMAP_HH_

#include <qwidget.h>
#include <qpixmap.h>
#include <qevent.h>

#include "rcobject.h"

#include "config.h"

#include <map>
#include <iterator>

//#include <list>
#include <utility>
#include <set>

#include "spaceobject.h"

using namespace std;

DECLARE_FWD_PTR(SpaceTool);
class SpaceMap;
class ObjectRef;

#include "spacetools.h"

struct LevelCmp
{
  bool operator()(const SpaceObjectPtr& p1, const SpaceObjectPtr& p2) const;
};

struct LevelEqual
{
  typedef SpaceObjectPtr first_argument_type;
  typedef SpaceObjectPtr second_argument_type;
  typedef bool result_type;
  bool operator()(const SpaceObjectPtr& p1, const SpaceObjectPtr& p2) const;
};

class SpaceCase : public set<SpaceObjectPtr,LevelCmp>, public RefCountObject
{
public:
  SpaceCase(int pos, SpaceMap *parrent);
  void toggle_select();
  void select(bool s);
  pair<iterator,bool> insert(SpaceObjectPtr object);
  void repaint();
  void deleteItem(int pos);
  bool selected();
protected:
  bool m_selected;
  int pos;
  SpaceMap *smap;
};

DECLARE_PTR(SpaceCase);

class SpaceIterator
{
public:
  typedef forward_iterator_tag iterator_category;
  typedef pair<int,SpaceObjectPtr> value_type;
  typedef ptrdiff_t difference_type;
  typedef value_type* pointer;
  typedef value_type& reference;

  SpaceIterator();
  // If begin is false then we want the end ...
  SpaceIterator(const map<int,SpaceCasePtr>& smap, bool begin = true);
  SpaceIterator(const SpaceIterator& copy);
  pair<int,SpaceObjectPtr> operator*();
  //pair<int,SpaceObjectPtr>* operator->();
  SpaceIterator& operator++();
  SpaceIterator operator++(int);
  bool operator==(const SpaceIterator& other) const;
  bool operator!=(const SpaceIterator& other) const;
  SpaceIterator& operator=(const SpaceIterator& copy);
protected:
  map<int, SpaceCasePtr>::const_iterator main_it, end;
  SpaceCase::iterator local_it;
};

class SpaceMap : public QWidget
{
  Q_OBJECT

  friend class SpaceCase;
public:
  typedef map<int,SpaceCasePtr> case_map_type;
  typedef map<int,SpaceCasePtr>::iterator case_iterator;
  typedef map<int,SpaceCasePtr>::const_iterator case_const_iterator;
  SpaceMap(int w = 1, int h = 1, QWidget * parent=0, const char * name=0, WFlags f=0);
  void setRef( ObjectRef* oref );
  int caseId(QPoint p) const;
  QPoint casePos(int id) const;
  QRect caseRect(int id) const;
  QRect caseRect(QPoint p) const;
  QPixmap* getPixmap();
  SpaceCasePtr getCase(int pos);
  void setTool(SpaceToolPtr tool);
  void setCase(SpaceCasePtr c);
  SpaceIterator begin() const;
  SpaceIterator end() const;
  case_iterator case_begin();
  case_iterator case_end();
  case_const_iterator case_begin() const;
  case_const_iterator case_end() const;
  int width() const;
  int height() const;
  void reset(int w, int h);
  void setSmall(bool small);
  void toggleSize();
  ObjectRef* getObjRef() { return objRef; }

public slots:
  void DoNothing() {}
  void deleteObject(int ptr);
  void itemParameter(int id);

protected:
  void paintEvent(QPaintEvent *event);
  void mouseMoveEvent(QMouseEvent *event);
  void mousePressEvent(QMouseEvent *event);
  void mouseReleaseEvent(QMouseEvent *event);
  void initGraphic();
  void initVars( bool initTool = true );

  int baseSize;
  QPoint curPos;
  int m_width, m_height;
  QPixmap buffer;
  map<int, SpaceCasePtr> smap;
  int currentCase, pressedCase;
  QRect rectCase;
  bool pressed;
  SpaceToolPtr tool;
  SpaceCasePtr curCase;
  bool isSmall;
  ObjectRef* objRef;
};

#endif
