/***************************************************************************
                          spaceref.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SPACEREF_HH_
#define _SPACEREF_HH_

#include "config.h"

#include <qstring.h>

#include <map>

#include <utility>

using namespace std;

class ReferenceCenter;

class Referencer
{
public:
  typedef int reference_id;
  static const reference_id null_reference;
  Referencer(ReferenceCenter* rc) : refCenter(rc) { }
  Referencer(const Referencer& copy) : refCenter(copy.refCenter) { }
  virtual ~Referencer() { };
  virtual void eraseReference( reference_id ref ) = 0;
  void changeReferenceCenter(ReferenceCenter* rc) {refCenter = rc;}
  ReferenceCenter* refCenter;

  void bind( reference_id ref );
  void unbind( reference_id ref );

  QString getName( reference_id ref ) const;
  Referencer::reference_id getId( const QString& name ) const;

  QString getName_nofail( reference_id ref ) const;
  reference_id getId_nofail( const QString& name );

  void renameRef( Referencer::reference_id ref, const QString& new_name );

};

class ReferenceCenter
{
public:
  struct Reference
  {
    Reference( QString n = "", Referencer *r = NULL, int b = 0 )
      : name( n )
      , referencer( r )
      , bound( b )
    { }
    QString name;
    Referencer *referencer;
    int bound;
  };
  ReferenceCenter();
  typedef map<Referencer::reference_id, Reference> references_type;
  bool exists(const QString& name) const;
  bool exists( Referencer::reference_id ref ) const;
  int referenced( Referencer::reference_id ref ) const;
  // These methods assume there is no conflict ...
  Referencer::reference_id createRef(const QString& name, Referencer* r);
  void eraseRef( Referencer::reference_id ref );
  void renameRef( Referencer::reference_id ref, const QString& newname );
  // These methods assume also the reference exists ...
  void bind( Referencer::reference_id ref );
  void unbind( Referencer::reference_id ref );
  QString getName( Referencer::reference_id ref ) const;
  Referencer::reference_id getId( const QString& name ) const;

  // The nofail methods are for used when building from file and when binding
  // may occure before creation
  Referencer::reference_id getId_nofail( const QString& name );
  QString getName_nofail( Referencer::reference_id ref ) const;
  void bind_nofail( Referencer::reference_id ref );
  //void unbind_nofail( Referencer::reference_id ref );
  bool exists_nofail( Referencer::reference_id ref );
  bool exists_nofail( const QString& name );

  bool checkFailsafe();

  QString createName() const;
  void clear();

  // TODO erase all unreferenced references
  // void clean();
protected:

  Referencer::reference_id createId();
  map<QString, Referencer::reference_id> existing_string, failed_string;
  references_type references, failsafe;
  Referencer::reference_id current_id;
};

struct RefException
{
  RefException( const QString& str ) : name( str ) {}
  QString name;
};

struct StillRefException : public RefException
{
  StillRefException( const QString& str ) : RefException( str ) {}
};

struct InexistingRefException : public RefException
{
  InexistingRefException( Referencer::reference_id ref ) : RefException( QString::number( ref ) ) {}
  InexistingRefException( const QString& name ) : RefException( name ) {}
};

struct NameConflictException : public RefException
{
  NameConflictException( const QString& str ) : RefException( str ) {}
};

struct CmpRefsByName
{
  typedef Referencer::reference_id first_argument_type;
  typedef Referencer::reference_id second_argument_type;
  typedef bool result_type;
  CmpRefsByName( ReferenceCenter *rc ) : refCenter( rc ) {}
  CmpRefsByName( const CmpRefsByName& copy ) : refCenter( copy.refCenter ) {}

  bool operator()( Referencer::reference_id e1, Referencer::reference_id e2 ) const
    {
      QString name1 = refCenter->getName( e1 );
      QString name2 = refCenter->getName( e2 );
      return name1 < name2;
    }

  ReferenceCenter *refCenter;
};

#endif
