/***************************************************************************
                          spacetools.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "spacetools.h"
#include "global.h"

#include <qpopupmenu.h>

SpaceTool::SpaceTool(string n)
  : name(n)
{ }

string SpaceTool::getName()
{
  return name;
}

FloorTool::FloorTool()
  : SpaceTool("Floor")
{ }

bool FloorTool::apply(int Case, SpaceMap& smap)
{
  SpaceCasePtr c = smap.getCase(Case);
  c->insert(new FloorObject( smap.getObjRef() ));
  return true;
}

const QPixmap& FloorTool::getImage()
{
  FloorObject o(NULL);
  return o.getImage(true);
}

DoorTool::DoorTool()
  : SpaceTool("Door")
{ }

bool DoorTool::apply(int Case, SpaceMap& smap)
{
  QPoint pt = smap.casePos(Case);
  int ptId;
  SpaceCasePtr Side;
  SpaceObjectPtr object;
  ptId = smap.caseId(pt+QPoint(1,0));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new DoorObject(smap.getObjRef(), true);
	}
    }
  ptId = smap.caseId(pt+QPoint(-1,0));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new DoorObject(smap.getObjRef(), true);
	}
    }
  ptId = smap.caseId(pt+QPoint(0,1));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new DoorObject(smap.getObjRef(), false);
	}
    }
  ptId = smap.caseId(pt+QPoint(0,-1));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new DoorObject(smap.getObjRef(), false);
	}
    }
  if(object.isNull())
    {
      object = new DoorObject(smap.getObjRef(), true);
    }
  SpaceCasePtr c = smap.getCase(Case);
  c->insert(object);
  return true;
}

const QPixmap& DoorTool::getImage()
{
  return DoorObject(NULL, true).getImage(true);
}

SelectTool::SelectTool()
  : SpaceTool("Selection")
{ }

bool SelectTool::apply(int Case, SpaceMap& smap)
{
  SpaceCasePtr c = smap.getCase(Case);
  c->toggle_select();
  return true;
}

QPixmap *SelectTool::image = NULL;

const QPixmap& SelectTool::getImage()
{
  if(image == NULL)
    {
      image = new QPixmap(getEditorDir() + "select.png");
    }
  return *image;
}

QPixmap* EraseTool::image = NULL;

EraseTool::EraseTool()
  : SpaceTool("Erase")
{
}

bool EraseTool::apply(int Case, SpaceMap& smap)
{
  SpaceCasePtr c = smap.getCase(Case);
  c->clear();
  c->repaint();
  return true;
}

const QPixmap& EraseTool::getImage()
{
  if(image == NULL)
    {
      image = new QPixmap(getEditorDir() + "erase.png");
    }
  return *image;
}

QPixmap* DeleteOneTool::image = NULL;

DeleteOneTool::DeleteOneTool()
  : SpaceTool("Delete the upper item")
{
}

bool DeleteOneTool::apply(int Case, SpaceMap& smap)
{
  SpaceCasePtr c = smap.getCase(Case);
  SpaceCase::iterator it = c->end();
  if(it != c->begin())
    {
      it--;
      c->erase(it);
      c->repaint();
    }
  return true;
}

const QPixmap& DeleteOneTool::getImage()
{
  if(image == NULL)
    {
      image = new QPixmap(getEditorDir() + "delete_one.png");
    }
  return *image;
}

EntryTool::EntryTool()
  : SpaceTool("Entry zone")
{ }

bool EntryTool::apply(int Case, SpaceMap& smap)
{
  QPoint pt = smap.casePos(Case);
  int ptId;
  SpaceCasePtr Side;
  SpaceObjectPtr object;
  ptId = smap.caseId(pt+QPoint(1,0));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new EntryObject(smap.getObjRef(), RIGHT);
	}
    }
  ptId = smap.caseId(pt+QPoint(-1,0));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new EntryObject(smap.getObjRef(), LEFT);
	}
    }
  ptId = smap.caseId(pt+QPoint(0,1));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new EntryObject(smap.getObjRef(), DOWN);
	}
    }
  ptId = smap.caseId(pt+QPoint(0,-1));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new EntryObject(smap.getObjRef(), UP);
	}
    }
  if(object.isNull())
    {
      object = new EntryObject(smap.getObjRef(), DOWN);
    }
  SpaceCasePtr c = smap.getCase(Case);
  c->insert(object);
  return true;
}

const QPixmap& EntryTool::getImage()
{
  return EntryObject(NULL,DOWN).getImage(true);
}

BulkHeadTool::BulkHeadTool()
  : SpaceTool("BulkHead")
{ }

bool BulkHeadTool::apply(int Case, SpaceMap& smap)
{
  QPoint pt = smap.casePos(Case);
  int ptId;
  SpaceCasePtr Side;
  SpaceObjectPtr object;
  ptId = smap.caseId(pt+QPoint(1,0));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new BulkHeadObject(smap.getObjRef(), true);
	}
    }
  ptId = smap.caseId(pt+QPoint(-1,0));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new BulkHeadObject(smap.getObjRef(), true);
	}
    }
  ptId = smap.caseId(pt+QPoint(0,1));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new BulkHeadObject(smap.getObjRef(), false);
	}
    }
  ptId = smap.caseId(pt+QPoint(0,-1));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new BulkHeadObject(smap.getObjRef(), false);
	}
    }
  if(object.isNull())
    {
      object = new BulkHeadObject(smap.getObjRef(), true);
    }
  SpaceCasePtr c = smap.getCase(Case);
  c->insert(object);
  return true;
}

const QPixmap& BulkHeadTool::getImage()
{
  return BulkHeadObject(NULL, true).getImage(true);
}

ExitTool::ExitTool()
  : SpaceTool("Exit zone")
{ }

bool ExitTool::apply(int Case, SpaceMap& smap)
{
  QPoint pt = smap.casePos(Case);
  int ptId;
  SpaceCasePtr Side;
  SpaceObjectPtr object;
  ptId = smap.caseId(pt+QPoint(1,0));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new ExitObject(smap.getObjRef(), RIGHT);
	}
    }
  ptId = smap.caseId(pt+QPoint(-1,0));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new ExitObject(smap.getObjRef(), LEFT);
	}
    }
  ptId = smap.caseId(pt+QPoint(0,1));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new ExitObject(smap.getObjRef(), DOWN);
	}
    }
  ptId = smap.caseId(pt+QPoint(0,-1));
  if(ptId != -1)
    {
      Side = smap.getCase(ptId);
      if(!Side->empty())
	{
	  object = new ExitObject(smap.getObjRef(), UP);
	}
    }
  if(object.isNull())
    {
      object = new ExitObject(smap.getObjRef(), DOWN);
    }
  SpaceCasePtr c = smap.getCase(Case);
  c->insert(object);
  return true;
}

const QPixmap& ExitTool::getImage()
{
  return ExitObject(NULL, DOWN).getImage(true);
}

QPixmap *ExtensibleTool::image = NULL;

ExtensibleTool::ExtensibleTool()
  : SpaceTool("Extensible object")
{ }

const QPixmap& ExtensibleTool::getImage()
{
  if(image == NULL)
    {
      image = new QPixmap(getEditorDir() + "extensible.png");
    }
  return *image;
}

bool ExtensibleTool::apply(int Case, SpaceMap& smap)
{
  SpaceCasePtr c = smap.getCase(Case);
  c->insert(new ExtensibleObject( smap.getObjRef() ));
  return true;
}

