/***************************************************************************
                          spacetools.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SPACETOOLS_HH_
#define _SPACETOOLS_HH_

#include <string>

#include "config.h"
#include "rcobject.h"

#include "spacemap.h"
#include "spaceobject.h"

using namespace std;

class SpaceTool : public RefCountObject
{
public:
  SpaceTool(string n = "");
  virtual ~SpaceTool() {}
  // Return true if the tool can be used while gliding the mouse ...
  virtual bool apply(int Case, SpaceMap& map) = 0;
  virtual string getName();
  virtual const QPixmap& getImage() = 0;
protected:
  string name;
};

DECLARE_PTR(SpaceTool);

class FloorTool : public SpaceTool
{
public:
  FloorTool();
  bool apply(int Case, SpaceMap& map);
  virtual const QPixmap& getImage();
};

class DoorTool : public SpaceTool
{
public:
  DoorTool();
  bool apply(int Case, SpaceMap& map);
  virtual const QPixmap& getImage();
};

class SelectTool : public SpaceTool
{
public:
  SelectTool();
  bool apply(int Case, SpaceMap& map);
  virtual const QPixmap& getImage();
protected:
  static QPixmap* image;
};

class EraseTool : public SpaceTool
{
public:
  EraseTool();
  bool apply(int Case, SpaceMap& map);
  virtual const QPixmap& getImage();
protected:
  static QPixmap* image;
};

class DeleteOneTool : public SpaceTool
{
public:
  DeleteOneTool();
  bool apply(int Case, SpaceMap& map);
  virtual const QPixmap& getImage();
protected:
  static QPixmap* image;
};

class EntryTool : public SpaceTool
{
public:
  EntryTool();
  bool apply(int Case, SpaceMap& map);
  virtual const QPixmap& getImage();
};

class BulkHeadTool : public SpaceTool
{
public:
  BulkHeadTool();
  bool apply(int Case, SpaceMap& map);
  virtual const QPixmap& getImage();
};

class ExitTool : public SpaceTool
{
public:
  ExitTool();
  bool apply(int Case, SpaceMap& map);
  virtual const QPixmap& getImage();
};

class ExtensibleTool : public SpaceTool
{
public:
  ExtensibleTool();
  bool apply(int Case, SpaceMap& map);
  virtual const QPixmap& getImage();
protected:
  static QPixmap* image;
};

#endif
