/***************************************************************************
                          spacezones.h  -  description
                             -------------------
    copyright            : (C) 2003 by Pierre Barbier de Reuille
    email                : harbort@nerim.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SPACEZONES_HH_
#define _SPACEZONES_HH_

#include "config.h"
#include "rcobject.h"

#include "spacemap.h"
#include "spaceref.h"

#include <qstring.h>
#include <qdom.h>

#include <list>
#include <map>
#include <set>

using namespace std;

class SpaceZones : public RefCountObject, public Referencer
{
public:
  typedef map<Referencer::reference_id, set<int> > zones_type;
  typedef map<Referencer::reference_id, set<Referencer::reference_id> > zone_sets_type;

  SpaceZones(const SpaceZones& copy);
  SpaceZones(ReferenceCenter* rc);
  void setMap(SpaceMap* smap);
  // All thiese functions throw an exception for error ...
  Referencer::reference_id setZone(const QString& zone_name, const set<int>& zone);
  Referencer::reference_id setZone(const QString& zone_name);
  // return the ref of the created zone
  Referencer::reference_id setZone(const QDomElement& xml);

  Referencer::reference_id setZoneSet(const QString& zone_set_name, const set<Referencer::reference_id>& zone_set);
  Referencer::reference_id setZoneSet(const QString& zone_set_name);
  // return the name of the create zone set
  Referencer::reference_id setZoneSet(const QDomElement& xml);

  set<int> getZone( Referencer::reference_id ref );
  list<set<int> > getZoneSet( Referencer::reference_id ref );

  void setDoc(QDomDocument* doc);
  QDomElement getXmlElement( Referencer::reference_id ref ) const;

  // Method for the referencer :
  virtual void eraseReference( Referencer::reference_id ref );

  void clear();

  // Remove every unreference zones and zone sets
  void clean();

  zones_type zones;
  zone_sets_type zone_sets;
protected:
  SpaceMap* smap;
  QDomDocument* doc;
};

struct saveCase
{
  saveCase(QDomElement& r, const SpaceMap& m, QDomDocument& d);
  void operator()(int pos);
protected:
  QDomElement& root;
  const SpaceMap& smap;
  QDomDocument& doc;
};

struct saveRef
{
  saveRef(const QString& t, QDomElement& r, QDomDocument& d);
  void operator()(const QString& ref_name);
protected:
  QDomElement& root;
  QDomDocument& doc;
  QString type;
};

struct loadCases
{
  loadCases(const SpaceMap& smap, SpaceZones& zones);
  set<int> operator()(QDomElement& root);
protected:
  const SpaceMap& smap;
  SpaceZones& zones;
};

#endif
